# -*- coding: utf-8 -*-
from gi.repository import Gtk
import difflib,re
DEBUG=False
def debug (*args,**kwargs):
    if DEBUG: print args,kwargs

class TooManyChanges (Exception):
    def __init__ (self, value):
        self.value=value
    def __str__(self):
        return repr(self.value)

class UndoableObject:
    """An UndoableObject. This must provide an action, an inverse and a history. Alternatively,
    it can supply get_reapply_action_args, which will allow the action to be "reapplied" to new
    arguments (for example, if the action is setting a text attribute, reapply might set that attribute
    for the currently highlighted text)."""
    def __init__ (self, action, inverse, history,
                  action_args=None,
                  undo_action_args=None,
                  get_reapply_action_args=None,
                  get_reundo_action_args=None,
                  reapply_name=None,
                  reundo_name=None,
                  is_undo=False):
        if not action_args: action_args=[]
        if not undo_action_args: undo_action_args=[]
        self.history=history
        self.action=action
        self.inverse_action=inverse
        self.get_reapply_action_args = get_reapply_action_args
        self.undo_action_args = undo_action_args
        self.get_reundo_action_args=get_reundo_action_args
        self.reapply_name = reapply_name
        self.reundo_name = reundo_name
        self.is_undo=is_undo
        if self.get_reapply_action_args:
            self.reapplyable = True
        else:
            self.reapplyable = False
        self.action_args = action_args

    def perform (self):
        self.action(*self.action_args)
        self.history.append(self)

    def inverse (self):        
        u = UndoableObject(self.inverse_action, self.action, self.history,
                           action_args=self.undo_action_args,
                           undo_action_args=self.action_args,
                           get_reapply_action_args=self.get_reundo_action_args,
                           get_reundo_action_args=self.get_reapply_action_args,
                           is_undo=not self.is_undo)
        self.history.remove(self)
        u.perform()

    def reapply (self):
        if self.get_reapply_action_args:
            args,undoargs = self.get_reapply_action_args()
            u = UndoableObject(self.action,self.inverse_action,self.history,
                               action_args=args,
                               undo_action_args=undoargs,
                               get_reapply_action_args=self.get_reapply_action_args,
                               reapply_name=self.reundo_name, reundo_name=self.reapply_name)
            u.perform()

class UndoableTextChange (UndoableObject):
    def __init__ (self, set_text_action, history, initial_text="",text="",txt_id=None):
        self.txt_id = txt_id
        self.blob_matcher = re.compile('\s+\S+\s+')
        self.initial_text = initial_text
        self.text = text
        self._set_text = set_text_action
        UndoableObject.__init__(self,lambda *args: self._set_text(self.text),lambda *args: self._set_text(self.initial_text),history)
        self.mode=self.determine_mode()
        try:
            self.cindex,self.clen = self.find_change(self.text)
        except TooManyChanges:
            self.cindex,self.clen = 0,0

    def determine_mode (self,text=None,initial_text=None):
        if not text: text=self.text
        if not initial_text: initial_text=self.initial_text
        if len(text) > len(initial_text):
            return 'add'
        elif len(text) < len(initial_text):
            return 'delete'

    def find_change (self, text2=None):
        if not self.mode:
            self.text = text2
            self.determine_mode()
        if not text2: text2=self.text
        if not hasattr(self,'sm'):
            self.sm = difflib.SequenceMatcher(None,self.initial_text,text2)
        else:
            self.sm.set_seq2(text2)
        blocks=self.sm.get_matching_blocks()
        # we only are interested in similar blocks at different positions
        # (which tell us where the changes happened).
        ch_blocks = filter(lambda x: x[0] != x[1] and x[2] != 0, blocks)
        if ch_blocks and len(ch_blocks)>1:
            raise TooManyChanges("More than one block changed: %s in %s"%(ch_blocks,text2))
        if ch_blocks:
            i,j,n = ch_blocks[0]
            change_length = j-i
            change_index = i
            return [change_index,change_length]
        else:
            if self.mode=='delete':
                return [len(self.initial_text),len(self.initial_text)-len(text2)]
            else: #self.mode=='add', we presume
                return [len(self.initial_text),len(text2)-len(self.initial_text)]

    def add_text (self, new_text):
        mode=self.determine_mode(new_text)
        contmode = self.determine_mode(new_text,self.text)
        if (mode == contmode == self.mode):
            try:
                cindex,clen = self.find_change(new_text)
            except TooManyChanges:
                self.new_action(new_text)
            else:
                if ((cindex==self.cindex) or
                    (self.mode=='delete' and cindex==self.cindex-(clen-self.clen))
                    ):
                    changed_text = new_text[cindex:cindex+clen]
                    if not self.blob_matcher.search(changed_text):
                        self.text = new_text
                        self.cindex,self.clen = cindex,clen
                        return
        self.new_action(new_text)

    def new_action (self, new_text):
        self.history.append(UndoableTextChange(self._set_text,self.history,
                                                   initial_text=self.text, text=new_text)
                            )

    def inverse (self):
        self._set_text(self.initial_text)
        u=UndoableTextChange(self._set_text,self.history,initial_text=self.text,
                             text=self.initial_text)
        u.is_undo=not self.is_undo
        self.history.remove(self)
        u.perform()

    def perform (self):
        self.history.append(self)

class UndoableTextContainer:
    def __init__ (self, container, history):
        self.history = history
        self.container = container
        self.setup_widgets()
        self.txt = self.get_text()
        self._setting = False

    def change_event_cb (self,*args):
        # if the last change is the same as us...
        if self._setting:
            return
        debug('change_event external',4)
        txt = self.get_text()
        if txt == self.txt: pass
        if len(self.history)>1 and hasattr(self.history[-1],'txt_id') and self.history[-1].txt_id==self.container:
            debug('adding text to previous change',0)
            self.change = self.history[-1]
            self.history[-1].add_text(txt)
        else:
            self.change=UndoableTextChange(self._set_text,
                                           self.history,
                                           initial_text=self.txt,
                                           text=txt,
                                           txt_id=self.container)
            debug('appending new change to history',0)
            self.history.append(self.change)
        self.txt = txt
        
    def setup_widgets (self): pass
    def get_text (self): raise NotImplementedError()

    def _set_text (self,txt):
        self._setting = True
        self.set_text(txt)
        self._setting = False
        
    def set_text (self,txt): raise NotImplementedError()

class UndoableEntry (UndoableTextContainer):
    def __init__ (self, entry,history):
        self.entry = entry
        self.get_text = self.entry.get_text
        UndoableTextContainer.__init__(self,self.entry,history)

    def setup_widgets (self): 
        self.entry.connect('changed',
                           self.change_event_cb
                           )
        
    def set_text (self, txt):
        index,length = self.change.find_change(txt)
        self.entry.set_text(txt)
        self.entry.grab_focus()
        self.entry.set_position(index + length)
        
class UndoableGenericWidget:
    def __init__ (self, widget, history, set_method='set_value',get_method='get_value', signal='changed',
                  pre_change_signal=None):
        self.w = widget
        self.set_method = set_method
        self.get_method = get_method
        self.set = getattr(self.w,self.set_method)
        self.get = getattr(self.w,self.get_method)
        self.history = history
        self.last_value = self.get()
        self.w.connect(signal,self.changecb)
        if pre_change_signal:
            self.w.connect(pre_change_signal,self.pre_changecb)

    def pre_changecb (self, *args):
        self.last_value = self.get()

    def changecb (self,*args):
        old_val = self.last_value
        new_val = self.get()
        if new_val != old_val:
            # We don't perform because we're being called after the action has happened.
            # We simply append ourselves to the history list.
            self.history.append(UndoableObject(lambda *args: self.set(new_val),
                                               lambda *args: self.set(old_val),
                                               self.history)
                                )
            self.last_value=new_val
        
class UndoableTextView (UndoableTextContainer):
    def __init__ (self, textview, history):
        self.tv = textview
        UndoableTextContainer.__init__(self,self.tv,history)
        
    def setup_widgets (self):
        self.buffer = self.tv.get_buffer()
        self.buffer.connect('changed',
                            self.change_event_cb)
        self.buffer.connect('apply-tag',self.change_event_cb)
        self.buffer.connect('remove-tag',self.change_event_cb)
        
    def set_text (self, text):        
        self.buffer.set_text(text)
        self.tv.grab_focus()
        try:
            index,length = self.change.find_change(text)
            self.buffer.place_cursor(self.buffer.get_iter_at_offset(index + length))
        except TooManyChanges:
            debug('WARNING: THAT SHOULDNT HAVE HAPPENED! Too many changes!',0)

    def get_text (self):
        return self.buffer.get_text(self.buffer.get_start_iter(),
                                    self.buffer.get_end_iter())        

class UndoHistoryList (list):
    """An UndoHistoryList."""
    def __init__ (self, undo_widget, redo_widget, reapply_widget=None, signal='activate'):
        self.undo_widget = undo_widget
        self.redo_widget = redo_widget
        self.reapply_widget = reapply_widget
        list.__init__(self)
        self.gui_update()
        if signal:
            if self.undo_widget: self.undo_widget.connect(signal,self.undo)
            if self.redo_widget: self.redo_widget.connect(signal,self.redo)
            if self.reapply_widget: self.reapply_widget.connect(signal,self.reapply)

    def undo (self, *args):
        index = -1
        if len(self) == 0: return False
        try:
            while self[index].is_undo:
                index = index - 1
        except:
            debug('All %s available action are .is_undo=True'%len(self),0)
            raise
        self[index].inverse()

    def redo (self, *args):
        if len(self) == 0: return False
        index = -1
        try:
            while not self[index].is_undo:
                index = index - 1
        except:
            debug('All %s available actions are is_undo=False'%len(self),0)
            raise
        self[index].inverse()

    def reapply (self, *args):
        debug('Reapplying',0)
        self[-1].reapply()

    def set_sensitive (self,w,val):
        debug('set_sensitive',0)
        if not w: return
        try:
            w.set_sensitive(val)
            debug('%s.set_sensitive succeeded'%w,0)
        except AttributeError:
            # 2.6 will give Gtk.Action a set_sensitive property, but for now...
            #if type(w)==Gtk.Action:
            for p in w.get_proxies():
                debug('setting %s sensitivity to %s'%(w,val),0)
                #p.set_sensitive(val)
                p.set_property('sensitive',val)

    def gui_update (self):
        debug('gui_update',0)
        if len(self) >= 1:
            undoables = [x.is_undo for x in self]
            if False in undoables:
                self.set_sensitive(self.undo_widget,True)
                debug('Sensitizing undo_widget',0)
            else:
                self.set_sensitive(self.undo_widget,False)
                debug('Desensizing undo_widget',0)
            if True in undoables:
                debug('Sensitizing redo_widget',0)
                self.set_sensitive(self.redo_widget,True)
            else:
                debug('Desensitizing redo widget',0)
                self.set_sensitive(self.redo_widget,False)
            if self[-1].reapplyable:
                debug('Sensitizing "reapply" widgets',0)
                self.set_sensitive(self.reapply_widget,True)
                if self[-1].reapply_name:
                    if type(self.reapply_widget)==Gtk.MenuItem:
                        alabel = self.reapply_widget.get_children()[0]
                        alabel.set_text_with_mnemonic(self[-1].reapply_name)
                        alabel.set_use_markup(True)
        else:
            debug('Nothing to undo, desensitizing widgets',0)
            self.set_sensitive(self.redo_widget,False)
            self.set_sensitive(self.undo_widget,False)
            self.set_sensitive(self.reapply_widget,False)
            
    def append (self,obj):
        debug('Appending %s'%obj,0)
        list.append(self,obj)
        self.gui_update()

    def remove (self,obj):
        debug('Removing %s'%obj,0)        
        list.remove(self,obj)
        self.gui_update()

    def clear (self):
        while self: list.remove(self,self[0])
        self.gui_update()
        

class MultipleUndoLists:
    """For tabbed interfaces and other places where it makes sense for one
    Action (menu item, etc.) to control multiple UndoLists (since presumably
    nobody wants to undo things they cannot see)"""
    def __init__ (self, undo_widget, redo_widget, reapply_widget=None, signal='activate',
                  get_current_id=None):
        self.undo_widget = undo_widget
        self.redo_widget = redo_widget
        self.reapply_widget = reapply_widget
        self.signal = signal
        self.get_current_id = get_current_id
        self.histories = {}
        if signal:
            if self.undo_widget: self.undo_widget.connect(signal,self.undo)
            if self.redo_widget: self.redo_widget.connect(signal,self.redo)
            if self.reapply_widget: self.reapply_widget.connect(signal,self.reapply)
        
        # attempts to implement the following programatically are failing me...
        # it feels awful to write each of these methods out here, but here goes...

    def __add__ (self,*args,**kwargs): return self.get_history().__add__(*args,**kwargs)
    def __contains__ (self,*args,**kwargs): return self.get_history().__contains__(*args,**kwargs)
    def __delitem__ (self,*args,**kwargs): return self.get_history().__delitem__(*args,**kwargs)
    def __delslice__ (self,*args,**kwargs): return self.get_history().__delslice__(*args,**kwargs)
    def __doc__ (self,*args,**kwargs): return self.get_history().__doc__(*args,**kwargs)
    def __eq__ (self,*args,**kwargs): return self.get_history().__eq__(*args,**kwargs)
    def __ge__ (self,*args,**kwargs): return self.get_history().__ge__(*args,**kwargs)
    def __getitem__ (self,*args,**kwargs): return self.get_history().__getitem__(*args,**kwargs)
    def __getslice__ (self,*args,**kwargs): return self.get_history().__getslice__(*args,**kwargs)
    def __gt__ (self,*args,**kwargs): return self.get_history().__gt__(*args,**kwargs)
    def __hash__ (self,*args,**kwargs): return self.get_history().__hash__(*args,**kwargs)
    def __iadd__ (self,*args,**kwargs): return self.get_history().__iadd__(*args,**kwargs)
    def __imul__ (self,*args,**kwargs): return self.get_history().__imul__(*args,**kwargs)
    def __iter__ (self,*args,**kwargs): return self.get_history().__iter__(*args,**kwargs)
    def __le__ (self,*args,**kwargs): return self.get_history().__le__(*args,**kwargs)
    def __len__ (self,*args,**kwargs): return self.get_history().__len__(*args,**kwargs)
    def __lt__ (self,*args,**kwargs): return self.get_history().__lt__(*args,**kwargs)
    def __mul__ (self,*args,**kwargs): return self.get_history().__mul__(*args,**kwargs)
    def __ne__ (self,*args,**kwargs): return self.get_history().__ne__(*args,**kwargs)
    def __new__ (self,*args,**kwargs): return self.get_history().__new__(*args,**kwargs)
    def __reduce__ (self,*args,**kwargs): return self.get_history().__reduce__(*args,**kwargs)
    def __reduce_ex__ (self,*args,**kwargs): return self.get_history().__reduce_ex__(*args,**kwargs)
    def __repr__ (self,*args,**kwargs): return self.get_history().__repr__(*args,**kwargs)
    def __rmul__ (self,*args,**kwargs): return self.get_history().__rmul__(*args,**kwargs)
    def __setitem__ (self,*args,**kwargs): return self.get_history().__setitem__(*args,**kwargs)
    def __setslice__ (self,*args,**kwargs): return self.get_history().__setslice__(*args,**kwargs)
    def __str__ (self,*args,**kwargs): return self.get_history().__str__(*args,**kwargs)
    def append (self,*args,**kwargs): return self.get_history().append(*args,**kwargs)
    def count (self,*args,**kwargs): return self.get_history().count(*args,**kwargs)
    def extend (self,*args,**kwargs): return self.get_history().extend(*args,**kwargs)
    def index (self,*args,**kwargs): return self.get_history().index(*args,**kwargs)
    def insert (self,*args,**kwargs): return self.get_history().insert(*args,**kwargs)
    def pop (self,*args,**kwargs): return self.get_history().pop(*args,**kwargs)
    def remove (self,*args,**kwargs): return self.get_history().remove(*args,**kwargs)
    def reverse (self,*args,**kwargs): return self.get_history().reverse(*args,**kwargs)
    def sort (self,*args,**kwargs): return self.get_history().sort(*args,**kwargs)
    def redo (self,*args,**kwargs): return self.get_history().redo(*args,**kwargs)
    def undo (self,*args,**kwargs): return self.get_history().undo(*args,**kwargs)
    def reapply (self,*args,**kwargs): return self.get_history().reapply(*args,**kwargs)

    def get_history (self):
        hid=self.get_current_id()
        if self.histories.has_key(hid):
            #debug('Returning history %s for id %s'%([repr(i) for i in self.histories[hid]],hid),0)
            return self.histories[hid]
        else:
            #debug('Creating new history for id %s'%hid,0)
            self.histories[hid]=self.make_history()
            return self.histories[hid]

    def make_history (self):
        return UndoHistoryList(self.undo_widget,self.redo_widget,None,None)

    def switch_context (self, hid):
        # set sensitivity for current context
        debug('switching context...',0)
        self.get_history().gui_update()
            
if __name__ == '__main__':
    #txt = raw_input('Text: ')
    #history = []
    #utc = UndoableTextChange(None,None,history,text=txt)
    #history.append(utc)
    #while txt:
    #    txt = raw_input('Text: ')
    #    history[-1].add_text(txt)
    from gi.repository import Gtk
    w = Gtk.Window()
    e = Gtk.Entry()
    tv = Gtk.TextView()
    ub = Gtk.Button(stock=Gtk.STOCK_UNDO)
    rb = Gtk.Button(stock=Gtk.STOCK_REDO)
    sc = Gtk.Button('show changes')
    vb = Gtk.VBox()
    bb = Gtk.HButtonBox()
    bb.add(ub)
    bb.add(rb)
    bb.add(sc)
    vb.add(bb)
    vb.add(e)
    vb.add(tv)
    w.add(vb)
    uhl = UndoHistoryList(ub,rb,signal='clicked')
    UndoableTextView(tv,uhl)
    UndoableEntry(e,uhl)
    w.show_all()
    w.connect('delete-event',lambda *args:Gtk.main_quit())
    def show_changes (*args):
        for c in uhl:
            print c,' initial: ',c.initial_text,' current: ',c.text
    ub.connect('clicked',lambda *args: debug('Undo clicked!',0))
    sc.connect('clicked',show_changes)
    rb.connect('clicked',lambda *args: debug('Redo clicked!',0))    
    Gtk.main()





# -*- coding: utf-8 -*-
#
# defaults.py.in sets many important default global variables
# used throughout the game. Note that this file is processed by
# automake to set prefix paths etc. Please keep defaults.py.in
# in sync between glchess and gnome-sudoku. 

import os
import sys
import errno
import locale
import gettext

from gi.repository import GLib

try:
    from defs import VERSION, PREFIX
except ImportError:
    PREFIX = "/usr"
    VERSION = "0.0.0"

root_dir = os.path.dirname(os.path.dirname(__file__))
if not os.path.exists(os.path.join(root_dir, "Makefile.am")):
    # Running in installed mode
    APP_DATA_DIR    = os.path.join(PREFIX, 'share')
    BASE_DIR        = os.path.join(APP_DATA_DIR, 'gnome-sudoku')
    IMAGE_DIR       = os.path.join(BASE_DIR, 'images')
    LOCALEDIR       = os.path.join(APP_DATA_DIR, 'locale')
    UI_DIR          = BASE_DIR
    PUZZLE_DIR      = os.path.join(BASE_DIR, 'puzzles')
else:
    # Running in uninstalled mode
    sys.path.insert(0, os.path.abspath(root_dir))
    APP_DATA_DIR    = os.path.join(root_dir, '../data') 
    IMAGE_DIR       = os.path.join(root_dir, '../images')
    LOCALEDIR       = os.path.join(APP_DATA_DIR, 'locale')
    UI_DIR          = os.path.join(root_dir, '../data')
    BASE_DIR        = os.path.join(root_dir, '../data')
    PUZZLE_DIR      = BASE_DIR

DOMAIN = 'gnome-games'
locale.bind_textdomain_codeset(DOMAIN, "UTF-8") # See Bug 608425
gettext.bindtextdomain(DOMAIN, LOCALEDIR)
gettext.textdomain(DOMAIN)
from gettext import gettext as _

APPNAME             = _("GNOME Sudoku")
APPNAME_SHORT       = _("Sudoku")
COPYRIGHT           = 'Copyright \xc2\xa9 2005-2008, Thomas M. Hinkle'
DESCRIPTION         = _('GNOME Sudoku is a simple Sudoku generator and player. Sudoku is a Japanese logic puzzle.\n\nGNOME Sudoku is a part of GNOME Games.')
AUTHORS             = ("Thomas M. Hinkle","John Stowers")
WEBSITE             = 'http://www.gnome.org/projects/gnome-games/'
WEBSITE_LABEL       = _('GNOME Games web site')
AUTO_SAVE           = True
MIN_NEW_PUZZLES     = 90

DATA_DIR = os.path.join(GLib.get_user_config_dir(),"gnome-sudoku/")




# -*- coding: utf-8 -*-
def rgb_to_hsv (r, g, b, maxval = 255):
    if type(r) == int:
        r = r/float(maxval)
    if type(g) == int:
        g = g/float(maxval)
    if type(b) == int:
        b = b/float(maxval)
    # Taken from
    # http://www.easyrgb.com/math.php?MATH=M20#text20
    var_min = min(r, g, b)
    var_max = max(r, g, b)
    delta = var_max - var_min
    v = var_max
    if delta == 0:
        # we're grey
        h = 0
        s = 0
    else:
        s = delta/var_max
        delta_r = ( ( (var_max - r) / 6) + (delta/2) ) / delta
        delta_g = ( ( (var_max - g) / 6) + (delta/2) ) / delta
        delta_b = ( ( (var_max - b) / 6) + (delta/2) ) / delta
        if (r == var_max):
            h = delta_b - delta_g
        elif g == var_max:
            h = (1.0/3)+delta_r-delta_b
        elif b == var_max:
            h = (2.0/3)+delta_g-delta_r
        if (h < 0):
            h += 1
        if (h > 1):
            h -= 1
    return h, s, v

def hsv_to_rgb (h, s, v):
    if s == 0:
        return v, v, v
    else:
        h = h*6
        if h == 6:
            h = 0
        i = int(h)
        c1 = v*(1 - s)
        c2 = v*(1 - s * ( h-i ) )
        c3 = v *(1 - s * (1 - (h - i) ) )
        if i == 0:
            r, g, b = v, c3, c1
        elif i == 1:
            r, g, b = c2, v, c1
        elif i == 2:
            r, g, b = c1, v, c3
        elif i == 3:
            r, g, b = c1, c2, v
        elif i == 4:
            r, g, b = c3, c1, v
        else:
            r, g, b = v, c1, c2
        return r, g, b

def rotate_hue (h, s, v, rotate_by = .25):
    h += rotate_by
    if h > 1.0:
        h = h-1.0
    return h, s, v

def rotate_hue_rgb (r, g, b, rotate_by = 0.25, maxval = 255):
    h, s, v = rgb_to_hsv(r, g, b, maxval = maxval)
    h, s, v = rotate_hue (h, s, v, rotate_by = rotate_by)
    return hsv_to_rgb(h, s, v)

def color_hex_to_float (hstr):
    hstr = hstr.strip('#')
    if len(hstr) == 6:
        r = hstr[:2]
        g = hstr[2:4]
        b = hstr[4:]
        maxval = 255
    elif len(hstr) == 3:
        r, g, b = hstr
        maxval = 15
    else:
        raise ValueError('%s is not a 6 or 3 digit color string'%hstr)
    r, g, b = int(r, 16), int(g, 16), int(b, 16)
    return r/float(maxval), g/float(maxval), b/float(maxval)



# -*- coding: utf-8 -*-
from gi.repository import Gtk,Gio
import cairo
import time
import os.path
import sudoku, gsudoku, saver, defaults
from gettext import gettext as _

def fit_squares_in_rectangle (width, height, n, margin = 0):
    """Optimally fit squares into a rectangle.

    Return number of columns, number of rows, and square size
    for the best fit of n squares into a rectangle of given width and height.

    Optionally include a margin along edges and between squares.
    """
    best_square_size = 0
    best_fit = None
    for n_across in range(1, int(n) + 1):
        # if there's a remainder, we need to add an extra row...
        # i.e. 5 / 2 = 2 remainder 1, which means with two rows we
        # would need three columns, not two.
        n_down = n / n_across + (n % n_across and 1 or 0)
        across_size = width - ((n_across + 1) * margin)
        across_size = across_size / n_across
        down_size = height - ((n_down + 1) * margin)
        down_size = down_size / n_down
        if across_size < down_size:
            square_size = across_size
        else:
            square_size = down_size
        if square_size > best_square_size:
            best_square_size = square_size
            best_fit = n_across, n_down
    if best_fit:
        return best_fit, best_square_size

class SudokuPrinter:
    def __init__ (self,
                  sudokus,
                  main_window,
                  margin = 50,
                  sudokus_per_page = 1,
                  ):
        self.drawn = False
        self.margin = margin
        self.sudokus_per_page = sudokus_per_page
        self.n_sudokus = len(sudokus)
        self.sudokus = sudokus
        self.print_op = Gtk.PrintOperation()
        self.print_op.connect( "begin-print", self.begin_print)
        self.print_op.connect("draw-page", self.draw_page)
        self.main_window = main_window

    def begin_print(self, operation, context):
        import math
        pages = int(math.ceil(self.n_sudokus / self.sudokus_per_page))
        operation.set_n_pages(pages)

    def draw_page(self, operation, context, page_nr):
        import sudoku_thumber

        margin = 25
        cr = context.get_cairo_context()
        width = context.get_width()
        height = context.get_height()

        best_fit, best_square_size = fit_squares_in_rectangle(width, height, self.sudokus_per_page, margin)

        start = int(page_nr * self.sudokus_per_page)
        sudokus_on_page = self.sudokus[start : start + int(self.sudokus_per_page)]

        left = margin
        top = margin
        pos = [1, 1]
        for sudoku in sudokus_on_page:
            if type(sudoku) == tuple:
                label = sudoku[1]
                sudoku = sudoku[0]
            else:
                label = ""

            cr.set_font_size(12)
            cr.select_font_face("", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
            cr.set_source_rgb(0, 0, 0)
            xbearing, ybearing, width, height, xadvance, yadvance = cr.text_extents(label)
            cr.move_to(left, top - height / 2)
            cr.show_text(label)

            if isinstance(sudoku, gsudoku.SudokuGameDisplay):
                sudoku = sudoku.grid

            sudoku_thumber.draw_sudoku (cr, sudoku.grid, None, best_square_size, left, top, for_printing = True)
            if pos[0] < best_fit[0]:
                left = left + best_square_size + margin
                pos[0]  += 1
            else:
                top = top + best_square_size + margin
                left = margin
                pos[0] = 1
                pos[1] += 1


def print_sudokus(*args, **kwargs):
    sp = SudokuPrinter(*args, **kwargs)
    res = sp.print_op.run(Gtk.PrintOperationAction.PRINT_DIALOG, sp.main_window)
    if res == Gtk.PrintOperationResult.ERROR:
        error_dialog = Gtk.MessageDialog(sp.main_window,
                                      Gtk.DialogFlags.DESTROY_WITH_PARENT,
                                      Gtk.MessageType.ERROR,
                                      Gtk.ButtonsType.CLOSE,
                                      "Error printing file:\n")
        error_dialog.connect("response", lambda w, id: w.destroy())
        error_dialog.show()

class GamePrinter:

    ui_file = os.path.join(defaults.UI_DIR, 'print_games.ui')

    def __init__ (self, sudoku_maker):
        self.sudoku_maker = sudoku_maker
        self.settings = Gio.Settings("org.gnome.gnome-sudoku")
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain(defaults.DOMAIN)
        self.builder.add_from_file(self.ui_file)
        # Set up toggles...
        for key, wname in [('mark-printed-as-played', 'markAsPlayedToggle'),
                          ('print-already-played-games', 'includeOldGamesToggle'),
                          ('print-easy', 'easyCheckButton'),
                          ('print-medium', 'mediumCheckButton'),
                          ('print-hard', 'hardCheckButton'),
                          ('print-very-hard', 'very_hardCheckButton'),
                          ]:
            setattr(self, wname, self.builder.get_object(wname))
            try:
                assert(getattr(self, wname))
            except:
                raise AssertionError('Widget %s does not exist' % wname)
            self.wrap_toggle(key, getattr(self, wname))
        self.sudokusToPrintSpinButton = self.builder.get_object('sudokusToPrintSpinButton')
        self.sudokusPerPageSpinButton = self.builder.get_object('sudokusPerPageSpinButton')
        for key, widg in [('print-multiple-sudokus-to-print', self.sudokusToPrintSpinButton.get_adjustment()),
                          ('sudokus-per-page', self.sudokusPerPageSpinButton.get_adjustment())
                         ]:
            self.wrap_adjustment(key, widg)
        self.dialog = self.builder.get_object('dialog')
        self.dialog.set_default_response(Gtk.ResponseType.OK)
        self.dialog.connect('response', self.response_cb)

    def wrap_toggle (self, key_name, action):
        action.set_active(self.settings.get_boolean(key_name))
        action.connect('toggled', self.set_boolean_key, key_name)

    def set_boolean_key (self, action, key_name):
        self.settings.set_boolean(key_name, action.get_active())

    def wrap_adjustment (self, key_name, action):
        action.set_value(self.settings.get_int(key_name))
        action.connect('value-changed', self.set_int_key, key_name)

    def set_int_key (self, action, key_name):
        self.settings.set_int(key_name, action.get_value())

    def response_cb (self, dialog, response):
        if response not in (Gtk.ResponseType.ACCEPT, Gtk.ResponseType.OK):
            self.dialog.hide()
            return
        # Otherwise, we're printing!
        levels = []
        for cat in sudoku.DifficultyRating.categories:
            if getattr(self,
                       cat.replace(' ', '_') + 'CheckButton'
                       ).get_active():
                levels.append(cat)
        if not levels:
            levels = sudoku.DifficultyRating.categories.keys()
        nsudokus = self.sudokusToPrintSpinButton.get_adjustment().get_value()
        sudokus = self.sudoku_maker.get_puzzles(
            nsudokus,
            levels,
            new = not self.includeOldGamesToggle.get_active()
            )
        # Convert floating point difficulty into a label string
        sudokus.sort(cmp = lambda a, b: cmp(a[1], b[1]))
        labels = {'easy': _('Easy'),
                  'medium': _('Medium'),
                  'hard': _('Hard'),
                  'very hard': _('Very hard')}
        sudokus = [(sudoku.sudoku_grid_from_string(puzzle),
                    "%s (%.2f)" % (labels[sudoku.get_difficulty_category(d)], d)
                    ) for puzzle, d in sudokus]
        sp = SudokuPrinter(sudokus,
                           self.dialog,
                           sudokus_per_page = self.sudokusPerPageSpinButton.get_adjustment().get_value())

        self.sudokus_printed = sudokus
        response = sp.print_op.run(Gtk.PrintOperationAction.PRINT_DIALOG, sp.main_window)

        if response   == Gtk.PrintOperationResult.ERROR:
            pass
        elif response == Gtk.PrintOperationResult.APPLY:
            if self.markAsPlayedToggle.get_active():
                for sud, lab in self.sudokus_printed:
                    jar = {}
                    jar['game'] = sud.to_string()
                    jar['printed'] = True
                    jar['printed_at'] = time.time()
                    tracker = saver.SudokuTracker()
                    tracker.finish_jar(jar)
            self.dialog.hide()

    def run_dialog (self):
        self.dialog.show()





# -*- coding: utf-8 -*-
import optparse
import defaults

parser = optparse.OptionParser(
    version=defaults.VERSION,
    option_list=[
    optparse.make_option("-v", const=True, action="store_const",
                         dest="debug", help="Print debug information",
                         default=False),
    optparse.make_option("-p", const=True, action="store_const",
                         dest="profile", help="Profile gnome-sudoku",
                         default=False),
    optparse.make_option("-w", const=True, action="store_const",
                         dest="walk", help="Step through program",
                         default=False),
    ]
    )

options, args = parser.parse_args()


# Make a lovely wrapper
if options.debug:
    def simple_debug (f):
        def _ (self, *args, **kwargs):
            print self.__class__, f.__name__, args, kwargs
            return f(self, *args, **kwargs)
        return _

elif options.walk:
    ff = []
    def simple_debug (f):
        def _ (self, *args, **kwargs):
            if (self.__class__, f.__name__) not in ff:
                print self.__class__, f.__name__, args, kwargs
                if raw_input('Hit return to step forward (hit i to ignore this function): ')=='i':
                    ff.append((self.__class__, f.__name__))
            return f(self, *args, **kwargs)
        return _


else:
    def simple_debug (f):
        return f



# -*- coding: utf-8 -*-
from gi.repository import Gtk,Gdk,GObject
import colors
import math
import random
import logging
from simple_debug import simple_debug
import sudoku
import number_box
import tracker_info

def gtkcolor_to_rgb (color):
    return (color.red   / float(2**16),
            color.green / float(2**16),
            color.blue  / float(2**16))

class SudokuNumberGrid (Gtk.AspectFrame):

    def __init__ (self, group_size = 9):
        self.table = Gtk.Table(rows = group_size, columns = group_size, homogeneous = True)
        self.group_size = group_size
        self.__entries__ = {}
        for x in range(self.group_size):
            for y in range(self.group_size):
                e = number_box.SudokuNumberBox(upper = self.group_size)
                e.x = x
                e.y = y
                self.table.attach(e, x, x+1, y, y+1,
                                  )
                self.__entries__[(x, y)] = e
        GObject.GObject.__init__(self, obey_child = False)
        self.set_shadow_type(Gtk.ShadowType.NONE)
        self.eb = Gtk.EventBox()
        self.eb.add(self.table)
        self.add(self.eb)
        self.table.set_row_spacings(1)
        self.table.set_col_spacings(1)
        box_side = int(math.sqrt(self.group_size))
        for n in range(1, box_side):
            self.table.set_row_spacing(box_side*n-1, 2)
            self.table.set_col_spacing(box_side*n-1, 2)
        self.table.set_border_width(2)
        self.show_all()

    def set_parent_for(self, parent):
        for entry in self.__entries__.values():
            entry.set_parent_win(parent)

    def set_timer(self, timer):
        for entry in self.__entries__.values():
            entry.set_timer(timer)

    def get_focused_entry (self):
        return self.table.focus_child

    def set_bg_color (self, color):
        try:
            if type(color) == str:
                color = Gdk.color_parse(color)
                color = Gdk.RGBA(color.red/65535.0, color.green/65535.0, color.blue/65535.0)
            else:
                color = Gdk.RGBA(*color)
        except:
            logging.critical("set_bg_color handed Bad color: %s" % color, exc_info=True)
            return

        self.eb.override_color(Gtk.StateFlags.NORMAL, color)
        self.eb.override_background_color(Gtk.StateFlags.NORMAL, color)
        self.table.override_color(Gtk.StateFlags.NORMAL, color)
        self.table.override_background_color(Gtk.StateFlags.NORMAL, color)

        for e in self.__entries__.values():
            e.override_background_color(Gtk.StateFlags.NORMAL, color)

class SudokuGameDisplay (SudokuNumberGrid, GObject.GObject):

    __gsignals__ = {
        'focus-changed':(GObject.SignalFlags.RUN_LAST, None, ()),
        'puzzle-finished':(GObject.SignalFlags.RUN_LAST, None, ())
        }

    do_highlight_cells = False

    @simple_debug
    def __init__ (self, grid = None, group_size = 9,
                  show_impossible_implications = False):
        group_size = int(group_size)
        self.hints = 0
        self.hint_square = None
        self.always_show_hints = False
        self.show_impossible_implications = show_impossible_implications
        self.impossible_hints = 0
        self.impossibilities = []
        self.trackers = {}
        self.tinfo = tracker_info.TrackerInfo()
        GObject.GObject.__init__(self)
        SudokuNumberGrid.__init__(self, group_size = group_size)
        self.setup_grid(grid, group_size)
        for e in self.__entries__.values():
            e.show()
            e.connect('undo-change', self.entry_callback, 'undo-change')
            e.connect('changed', self.entry_callback)
            e.connect('focus-in-event', self.focus_callback)
            e.connect('key-press-event', self.key_press_cb)
        self.connect('focus-changed', self.highlight_cells)

    def key_press_cb (self, widget, event):
        key = Gdk.keyval_name(event.keyval)
        dest = self.go_around(widget.x, widget.y, key)
        if dest:
            self.table.set_focus_child(self.__entries__[dest])

    def go_around (self, x, y, direction):
        '''return the coordinate if we should go to the other side of the grid.
        Or else return None.'''
        (limit_min, limit_max) = (0, self.group_size -1)
        if   (y, direction) == (limit_min, 'Up'):
            dest = (x, limit_max)
        elif (y, direction) == (limit_max, 'Down'):
            dest = (x, limit_min)
        elif (x, direction) == (limit_min, 'Left'):
            dest = (limit_max, y)
        elif (x, direction) == (limit_max, 'Right'):
            dest = (limit_min, y)
        else:
            return None
        return dest

    @simple_debug
    def focus_callback (self, e, event):
        self.focused = e
        self.emit('focus-changed')

    def get_highlight_colors (self):
        bg = self.get_style_context().get_background_color(Gtk.StateFlags.SELECTED)
        default_color = (bg.red, bg.green, bg.blue)
        hsv = colors.rgb_to_hsv(*default_color)
        box_s = hsv[1]
        box_v = hsv[2]
        if box_v < 0.5:
            box_v = box_v * 2
        if box_s > 0.75:
            box_s = box_s * 0.5
        else:
            box_s = box_s * 1.5
            if box_s > 1:
                box_s = 1.0
        self.box_color = colors.hsv_to_rgb(hsv[0], box_s, box_v)
        self.box_and_row_color = colors.rotate_hue_rgb(*self.box_color, **{'rotate_by': 0.33 / 2})
        self.row_color = colors.rotate_hue_rgb(*self.box_color, **{'rotate_by': 0.33})
        self.col_color = colors.rotate_hue_rgb(*self.box_color, **{'rotate_by': 0.66})
        self.box_and_col_color = colors.rotate_hue_rgb(*self.box_color, **{'rotate_by': 1.0 - (0.33 / 2)})

    def toggle_highlight (self, val):
        self.do_highlight_cells = val
        self.unhighlight_cells()
        if hasattr(self, 'focused') and self.focused:
            self.highlight_cells()

    def unhighlight_cells (self, *args):
        for e in self.__entries__.values():
            e.set_background_color(None)

    def highlight_cells (self, *args):
        if not self.do_highlight_cells:
            return
        self.unhighlight_cells()
        if not hasattr(self, 'box_color'):
            self.get_highlight_colors()
        my_x, my_y = self.focused.x, self.focused.y

        # col_coords can sometimes be null.
        if not hasattr(self.grid, 'col_coords'):
            return

        for x, y in self.grid.col_coords[my_x]:
            if (x, y) != (my_x, my_y):
                self.__entries__[(x, y)].set_background_color(self.col_color)
        for x, y in self.grid.row_coords[my_y]:
            if (x, y) != (my_x, my_y):
                self.__entries__[(x, y)].set_background_color(self.row_color)
        for x, y in self.grid.box_coords[self.grid.box_by_coords[(my_x, my_y)]]:
            if (x, y) != (my_x, my_y):
                e = self.__entries__[(x, y)]
                if x == my_x:
                    e.set_background_color(self.box_and_col_color)
                elif y == my_y:
                    e.set_background_color(self.box_and_row_color)
                else:
                    e.set_background_color(self.box_color)

    def animate_hint (self):
        if self.hint_animate_count % 2 == 0:
            color = (1.0, 0.0, 0.0)
        else:
            color = None
        self.hint_square.set_border_color(color)
        self.hint_animate_count += 1

        if self.hint_animate_count == 4:
            self.hint_square = None
            return False

        return True;

    def set_hint_square (self, square):
        if self.hint_square is not None:
            self.hint_square.set_border_color(None)
            GObject.source_remove(self.hint_timer)
            self.hint_timer = None

        if square is None:
            self.hint_square = None
        else:
            self.hint_square = self.__entries__[square]
            self.hint_animate_count = 0
            self.animate_hint()
            self.hint_timer = GObject.timeout_add(150, self.animate_hint)

    @simple_debug
    def show_hint (self):
        min_options = 10;
        squares = []
        for x in xrange(9):
            for y in xrange(9):
                if self.grid._get_(x, y) != 0:
                    continue
                n_options = len(self.grid.possible_values(x, y))
                if n_options < min_options:
                    squares = [(x, y)]
                    min_options = n_options
                elif n_options == min_options:
                    squares.append((x, y))

        if len(squares) != 0:
            self.set_hint_square(random.choice(squares))
            self.hints += 1

    def show_hint_for_entry (self, entry, interactive = False):
        if interactive:
            set_method = entry.set_note_text_interactive
        else:
            set_method = entry.set_note_text
        vals = self.grid.possible_values(entry.x, entry.y)
        vals = list(vals)
        vals.sort()
        if vals:
            ''.join([str(v) for v in vals])
            txt = ''.join([str(v) for v in vals])
            if txt != entry.get_text():
                set_method(bottom_text = txt, for_hint = True)
        elif not entry.get_text():
            if entry.get_text() != 'X':
                set_method(bottom_text = 'X', for_hint = True)
        else:
            set_method(bottom_text = "", for_hint = True)

    @simple_debug
    def reset_grid (self):
        '''Remove all untracked values from the grid

        This method is used to clear all untracked values from the grid for
        the undo processing.  The tracked values and notes are handled higher
        up by the caller.
        '''
        removed = []
        for x in range(self.group_size):
            for y in range(self.group_size):
                if not self.grid.virgin._get_(x, y):
                    e = self.__entries__[(x, y)]
                    val = e.get_value()
                    track = e.tracker_id
                    if val and track == tracker_info.NO_TRACKER:
                        removed.append((x, y, val))
                    self.remove(x, y)
        return removed

    def clear_notes (self, side = 'Both', tracker = None):
        '''Remove notes

        The list of notes removed by this function are returned in a list.
        The notes are returned in the format (x, y, (side, pos, tid, note)) where:
        x and y are the cell's coordinates
        side is either 'Top' or 'Bottom'
        pos is the index of the note within the notelist
        tid is the tracker id for the note
        note is the value of the note

        The side argument determines what notes get cleared as well as what
        notes get returned.
        'Both' - Clears both the top and bottom notes(default)
        'Top' - Clear only the top notes
        'Bottom' - Clear only the bottom notes
        'AutoHint' - Clear all bottom notes for all trackers
        'All' - Reset all notes

        For 'Top', 'Bottom', and 'Both', the tracker argument can be supplied
        to clear for a specific tracker.  Set tracker to None(default) to
        operate on just what is currently displayed.
        '''
        # Storage for removed notes
        removed = []
        for x in range(self.group_size):
            for y in range(self.group_size):
                e = self.__entries__[(x, y)]
                if side in ['Top', 'Both']:
                    if tracker == None:
                        top_display_list = e.get_note_display(e.top_note_list)[0]
                    else:
                        top_display_list = e.get_note_display(e.top_note_list, tracker, False)[0]
                    for offset, (notelist_index, tracker_id, note) in enumerate(top_display_list):
                        removed.append((x, y, ('Top', notelist_index, tracker_id, note)))
                        del e.top_note_list[notelist_index - offset]
                if side in ['Bottom', 'Both']:
                    if tracker == None:
                        bottom_display_list = e.get_note_display(e.bottom_note_list)[0]
                    else:
                        bottom_display_list = e.get_note_display(e.bottom_note_list, tracker, False)[0]
                    for offset, (notelist_index, tracker_id, note) in enumerate(bottom_display_list):
                        removed.append((x, y, ('Bottom', notelist_index, tracker_id, note)))
                        del e.bottom_note_list[notelist_index - offset]
                if side == 'All':
                    for notelist_index, (tracker_id, note) in enumerate(e.top_note_list):
                        removed.append((x, y, ('Top', notelist_index, tracker_id, note)))
                    e.top_note_list = []
                if side in ['All', 'AutoHint']:
                    for notelist_index, (tracker_id, note) in enumerate(e.bottom_note_list):
                        removed.append((x, y, ('Bottom', notelist_index, tracker_id, note)))
                    e.bottom_note_list = []
        # Redraw the notes
        self.update_all_notes()
        return removed

    def apply_notelist(self, notelist, apply_tracker = False):
        '''Re-apply notes

        Re-apply notes that have been removed with the clear_notes() function.
        The apply_tracker argument is used for the "Apply Tracker" button
        functionality, which requires the history to be updated.
        '''
        for x, y, (side, notelist_index, tracker_id, note) in notelist:
            cell = self.__entries__[x, y]
            if apply_tracker:
                use_tracker = tracker_info.NO_TRACKER
                cell.emit('notes-about-to-change')
            else:
                use_tracker = tracker_id
            if side == 'Top':
                cell.top_note_list.insert(notelist_index, (use_tracker, note))
            if side == 'Bottom':
                cell.bottom_note_list.insert(notelist_index, (use_tracker, note))
            if apply_tracker:
                cell.emit('notes-changed')
                # When applying a tracker - update the notes to remove
                # duplicates from other trackers.
                if side == 'Top':
                    cell.trim_untracked_notes(cell.top_note_list)
                else:
                    cell.trim_untracked_notes(cell.bottom_note_list)
        # Redraw the notes
        self.update_all_notes()

    @simple_debug
    def blank_grid (self):
        '''Wipe out everything on the grid.

        This blanks all values, notes, tracked values, virgin values.  You end
        up with a blank grid ready for a new puzzle.
        '''
        for x in range(self.group_size):
            for y in range(self.group_size):
                e = self.__entries__[(x, y)]
                if e.get_value():
                    self.remove(x, y)
                e.set_read_only(False)
        for imp_cell in self.impossibilities:
            self.__entries__[imp_cell].set_text('')
        self.impossibilities = []
        self.grid = None
        self.clear_notes('All')
        self.tinfo.reset()

    @simple_debug
    def change_grid (self, grid, group_size):
        self.hints = 0
        self.impossible_hints = 0
        self.blank_grid()
        self.setup_grid(grid, group_size)

    @simple_debug
    def load_game (self, game):
        """Load a game.

        A game is simply a two lined string where the first line represents our
        virgin self and line two represents our game-in-progress.
        """
        self.blank_grid()
        if '\n' in game:
            virgin, in_prog = game.split('\n')
        else:
            virgin, in_prog = game, ''
        group_size = int(math.sqrt(len(virgin.split())))
        self.change_grid(virgin, group_size = group_size)
        # This int() will break if we go to 16x16 grids...
        if in_prog:
            values = [int(c) for c in in_prog.split()]
            for row in range(group_size):
                for col in range(group_size):
                    index = row * 9 + col
                    if values[index] and not self.grid._get_(col, row):
                        self.add_value(col, row, values[index])

    @simple_debug
    def setup_grid (self, grid, group_size):
        self.doing_initial_setup = True
        if isinstance(grid, sudoku.SudokuGrid):
            self.grid = sudoku.InteractiveSudoku(grid.grid, group_size = grid.group_size)
        else:
            self.grid = sudoku.InteractiveSudoku(grid, group_size = group_size)
        for x in range(group_size):
            for y in range(group_size):
                val = self.grid._get_(x, y)
                if val:
                    self.add_value(x, y, val)
        self.doing_initial_setup = False

    @simple_debug
    def entry_callback (self, widget, *args):
        if not widget.get_text():
            self.remove(widget.x, widget.y, *args)
            # Trackers need to be redisplayed on an undo
            if args and args[0] == 'undo-change':
                self.show_track()
        else:
            self.entry_validate(widget, *args)

    def update_all_hints (self):
        for x in range(self.group_size):
            for y in range(self.group_size):
                e = self.__entries__[(x, y)]
                if e.read_only:
                    pass
                elif e.get_text():
                    e.set_note_text(bottom_text = '')
                else:
                    self.show_hint_for_entry(e)

    def update_all_notes (self):
        '''Display the notes for all the cells

        The notes are context sensitive to the trackers.  This method displays
        all of the notes for the currently viewed selection.
        '''
        for x in range(self.group_size):
            for y in range(self.group_size):
                self.__entries__[(x, y)].show_note_text()

    @simple_debug
    def entry_validate (self, widget, *args):
        val = widget.get_value()
        if (args and args[0] == 'undo-change'):
            # When undoing from one value to another - remove the errors from
            # the previous value and add the new value to the proper tracker
            self.remove_error_highlight()
            self.add_value(widget.x, widget.y, val, widget.tracker_id)
        else:
            self.add_value(widget.x, widget.y, val)
        if self.grid.check_for_completeness():
            self.emit('puzzle-finished')

    def highlight_conflicts (self, x, y):
        '''highlight any squares that conflict with position x,y.

        Conflict resolution is taken care of completely within
        the InteractiveGrid class.  A list of conflicting cells
        are stored in InteractiveGrid.conflicts
        '''
        # Return if there are no conflicts for this cell
        if not self.grid.conflicts.has_key((x, y)):
            return
        # Highlight the current cell
        self.__entries__[(x, y)].set_error_highlight(True)
        # Then highlight any cells that conflict with it
        for coord in self.grid.conflicts[(x, y)]:
            self.__entries__[coord].set_error_highlight(True)

    def set_value(self, x, y, val):
        '''Sets value for position x, y to val.

        Calls set_text_interactive so the history list is updated.
        '''
        self.__entries__[(x, y)].set_text_interactive(str(val))

    @simple_debug
    def add_value (self, x, y, val, tracker = None):
        """Add value val at position x, y.

        If tracker is set, we track the value with it.  Otherwise,
        the current tracker is used(default).
        """
        # If the cell already has a value - remove it first.
        e = self.__entries__[(x, y)]
        if e.get_value():
            self.remove(x, y)
        # Explicitly specified tracker
        if tracker:
            # Only add it to the display when it's tracker is visible
            if tracker == tracker_info.NO_TRACKER or tracker == self.tinfo.showing_tracker:
                self.__entries__[(x, y)].set_value(val, tracker)
            # If the tracker isn't showing at the moment - add it as a trace
            if tracker != tracker_info.NO_TRACKER:
                self.tinfo.add_trace(x, y, val, tracker)
        else:
            # Add a trace(tracked value) if a tracker is selected
            if self.tinfo.current_tracker != tracker_info.NO_TRACKER:
                self.tinfo.add_trace(x, y, val)
            # Remove all tracked values(all traces) for the cell if the player
            # adds an untracked value
            else:
                self.tinfo.remove_trace(x, y)
            self.__entries__[(x, y)].set_value(val, self.tinfo.current_tracker)
            if self.doing_initial_setup:
                self.__entries__[(x, y)].set_read_only(True)
            else:
                self.__entries__[(x, y)].recolor(self.tinfo.current_tracker)
        # Add it to the underlying grid
        self.grid.add(x, y, val, True)
        # Highlight any conflicts that the new value creates
        self.highlight_conflicts(x, y)
        # Draw our entry
        self.__entries__[(x, y)].queue_draw()
        # Update all hints if we need to
        if self.always_show_hints and not self.doing_initial_setup:
            self.update_all_hints()
        if not self.doing_initial_setup:
            self.mark_impossible_implications(x, y)

    @simple_debug
    def remove (self, x, y, *args):
        """Remove x, y from our visible grid.

        *args is passed from the undo mechanism
        """
        e = self.__entries__[(x, y)]
        # Always call the grid's remove() for proper conflict resolution
        if self.grid:
            self.grid.remove(x, y)
            self.remove_error_highlight()
        # Remove it from the tracker.  When removing via undo, the trace
        # manipulation is handled at a higher level
        if not args or args[0] != 'undo-change':
            if e.tracker_id != tracker_info.NO_TRACKER:
                self.tinfo.remove_trace(x, y, e.tracker_id)
        # Reset the value and tracker id
        e.set_value(0, tracker_info.NO_TRACKER)
        # Update all hints if we need to
        if self.grid and self.always_show_hints and not self.doing_initial_setup:
            self.update_all_hints()
        if not self.doing_initial_setup:
            self.mark_impossible_implications(x, y)

    def remove_error_highlight (self):
        '''remove error highlight from [x, y] and also all errors caused by it

        Conflict resolution is now handled within the InteractiveSudoku class.
        If any conflicts were cleared on the last remove() then they are
        stored in grid.cleared_conflicts
        '''
        if not self.grid.cleared_conflicts:
            return
        for coord in self.grid.cleared_conflicts:
            linked_entry = self.__entries__[coord]
            linked_entry.set_error_highlight(False)

    @simple_debug
    def auto_fill (self):
        changed = self.grid.auto_fill()
        retval = []
        for coords, val in changed:
            self.add_value(coords[0], coords[1], val)
            retval.append((coords[0], coords[1], val))
        if self.grid.check_for_completeness():
            self.emit('puzzle-finished')
        return retval

    @simple_debug
    def auto_fill_current_entry (self):
        e = self.get_focused_entry()
        if not e:
            return
        filled = self.grid.auto_fill_for_xy(e.x, e.y)
        if filled and filled != -1:
            e.set_text_interactive('')
            e.set_text_interactive(str(filled[1]))

    def __set_impossible (self, coords, setting):
        '''Call set_impossible() on a grid entry

        This function is a helper for the 'Warn about unfillable squares'
        feature.  It only calls set_impossible() if the option is on to prevent
        a check against the show_impossible_implications flag elsewhere.  The
        return from this function indicates whether or not the cell "may"
        have been modified, which is basically the value of the option setting.
        '''
        if self.show_impossible_implications:
            self.__entries__[coords].set_impossible(setting)
        return self.show_impossible_implications

    def display_impossible_implications (self):
        '''Start X-marking cells that have no possible values
        '''
        self.show_impossible_implications = True
        for imp_cell in self.impossibilities:
            self.__set_impossible(imp_cell, True)
            self.impossible_hints += 1
        if self.always_show_hints:
            self.update_all_hints()

    def hide_impossible_implications (self):
        '''Stop X-marking cells that have no possible values
        '''
        for imp_cell in self.impossibilities:
            self.__set_impossible(imp_cell, False)
        self.show_impossible_implications = False
        if self.always_show_hints:
            self.update_all_hints()

    @simple_debug
    def mark_impossible_implications (self, x, y, check_conflicts = True):
        '''Mark cells with X if they have no possible values

        The hint this method provides can be turned on and off from the
        menu Tools->'Warn about unfillable squares' option.

        The check_conflicts parameter is for internal use only.  It is used as
        a one level recursion on conflicts that the original target is involved
        with.

        Impossibilities are tracked regardless of the user's option setting.
        This was done to allow the user to toggle the option mid-game and still
        behave properly.  Conditional X-marking of cells happens in the
        __set_impossible() function.
        '''
        # Make sure we have a grid to work with
        if not self.grid:
            return
        # Flag whether or not we need to update hints
        grid_modified = False
        # Find any new impossible cells based on calling cell
        implications = self.grid.find_impossible_implications(x, y)
        if implications:
            for imp_cell in implications:
                grid_modified = self.__set_impossible(imp_cell, True)
                # Add them to the list if they aren't there already...
                if not imp_cell in self.impossibilities:
                    self.impossibilities.append(imp_cell)
                    # But don't score it unless the option is on
                    if self.show_impossible_implications:
                        self.impossible_hints += 1
        # Reset the list of impossible cells ignoring the called cell. Use a
        # copy to iterate over, so items can be removed while looping.
        if self.impossibilities:
            for imp_cell in self.impossibilities[:]:
                if imp_cell == (x, y):
                    continue
                if self.grid.possible_values(*imp_cell):
                    self.impossibilities.remove(imp_cell)
                    grid_modified = self.__set_impossible(imp_cell, False)
                else:
                    grid_modified = self.__set_impossible(imp_cell, True)
        # If any conflicts have been cleared or created, mark any impossible
        # cells they may have caused or removed
        if check_conflicts:
            for xx, yy in self.grid.cleared_conflicts:
                self.mark_impossible_implications(xx, yy, False)
            if self.grid.conflicts.has_key((x, y)):
                for xx, yy in self.grid.conflicts[(x, y)]:
                    self.mark_impossible_implications(xx, yy, False)
        # Update the hints if we need to
        if grid_modified and self.always_show_hints:
            self.update_all_hints()

    def delete_by_tracker (self):
        '''Delete all cells tracked by the current tracker

        The values are deleted from the tracker as well as the visible grid.
        '''
        ret = []
        tracker = self.tinfo.get_tracker(self.tinfo.showing_tracker)
        if not tracker:
            return ret
        for (x, y), value in tracker.items():
            ret.append((x, y, value, self.tinfo.showing_tracker))
            self.remove(x, y)
            if self.grid and self.grid._get_(x, y):
                self.grid.remove(x, y)
        return ret

    def cover_track(self, hide = False):
        '''Hide the current tracker

        All tracked values are deleted from the display, but kept by the
        tracker.  Setting hide to True changes prevents anything but untracked
        values to be shown after the call.
        '''
        track = self.tinfo.get_tracker(self.tinfo.showing_tracker)
        if track:
            for coord in track.keys():
                self.__entries__[coord].set_value(0, tracker_info.NO_TRACKER)
                self.grid.remove(*coord)
                self.remove_error_highlight()
                self.mark_impossible_implications(*coord)
        if hide:
            self.tinfo.hide_tracker()
        # Update all hints if we need to
        if self.always_show_hints and not self.doing_initial_setup:
            self.update_all_hints()

    def show_track(self):
        '''Displays the current tracker items

        The values and notes for the currently showing tracker will be
        displayed
        '''
        track = self.tinfo.get_tracker(self.tinfo.showing_tracker)
        if not track:
            return
        for (x, y), value in track.items():
            self.__entries__[(x, y)].set_value(value, self.tinfo.showing_tracker)
            self.__entries__[(x, y)].recolor(self.tinfo.showing_tracker)
            # Add it to the underlying grid
            self.grid.add(x, y, value, True)
            # Highlight any conflicts that the new value creates
            self.highlight_conflicts(x, y)
            # Draw our entry
            self.__entries__[(x, y)].queue_draw()
            self.mark_impossible_implications(x, y)
        # Update all hints if we need to
        if self.always_show_hints and not self.doing_initial_setup:
            self.update_all_hints()

if __name__ == '__main__':
    window = Gtk.Window()
    window.connect('delete-event', Gtk.main_quit)

    def test_number_grid ():
        t = SudokuNumberGrid(4)
        window.add(t)
        t.__entries__[(0, 1)].set_color((0.0, 1.0, 0.0))
        t.__entries__[(0, 1)].set_value(4)
        t.__entries__[(1, 1)].set_error_highlight(True)
        t.__entries__[(1, 1)].set_value(1)
        t.__entries__[(2, 1)].set_color((0.0, 0.0, 1.0))
        t.__entries__[(2, 1)].set_error_highlight(True)
        t.__entries__[(2, 1)].set_value(2)
        t.__entries__[(3, 1)].set_color((0.0, 0.0, 1.0))
        t.__entries__[(3, 1)].set_error_highlight(True)
        t.__entries__[(3, 1)].set_error_highlight(False)
        t.__entries__[(3, 1)].set_value(3)
        t.__entries__[(3, 1)].set_note_text('234', '12')

    def reproduce_foobared_rendering ():
        from dialog_swallower import SwappableArea
        sgd = SudokuGameDisplay()
        sgd.set_bg_color('black')
        vb = Gtk.VBox()
        hb = Gtk.HBox()
        swallower = SwappableArea(hb)
        tb = Gtk.Toolbar()
        b = Gtk.ToolButton(stock_id = Gtk.STOCK_QUIT)
        b.connect('clicked', lambda x: window.hide() or Gtk.main_quit())
        tb.add(b)
        def run_swallowed_dialog (*args):
            md = MessageDialog(title = "Bar", label = "Bar", sublabel = "Baz "*12)
            swallower.run_dialog(md)
        b2 = Gtk.ToolButton(label = 'Dialog')
        b2.connect('clicked', run_swallowed_dialog)
        tb.add(b2)
        vb.pack_start(tb, fill = False, expand = False)
        vb.pack_start(swallower, padding = 12)
        window.add(vb)
        window.show_all()
        from gtk_goodies.dialog_extras import MessageDialog
        md = MessageDialog(title = "Foo", label = "Foo", sublabel = "Bar "*12)
        swallower.run_dialog(md)
        hb.pack_start(sgd, padding = 6)
        game = '''1 8 4 2 0 0 0 0 0
                  0 6 0 0 0 9 1 2 0
                  0 2 0 0 8 0 0 0 0
                  0 1 8 0 5 0 0 0 0
                  9 0 0 0 0 0 0 0 3
                  0 0 0 0 1 0 6 5 0
                  0 0 0 0 9 0 0 8 0
                  0 5 7 1 0 0 0 9 0
                  0 0 0 0 0 3 5 4 7'''
        sgd.change_grid(game, 9)

    def test_sudoku_game ():
        game = '''1 8 4 2 0 0 0 0 0
                  0 6 0 0 0 9 1 2 0
                  0 2 0 0 8 0 0 0 0
                  0 1 8 0 5 0 0 0 0
                  9 0 0 0 0 0 0 0 3
                  0 0 0 0 1 0 6 5 0
                  0 0 0 0 9 0 0 8 0
                  0 5 7 1 0 0 0 9 0
                  0 0 0 0 0 3 5 4 7'''
        sgd = SudokuGameDisplay(game)
        sgd.set_bg_color('black')
        window.add(sgd)
        window.show_all()

#    test_number_grid()
#    reproduce_foobared_rendering()
    test_sudoku_game()
    window.show_all()
    Gtk.main()



# -*- coding: utf-8 -*-
from gi.repository import GObject
import time

class ActiveTimer (GObject.GObject):
    """A timer to keep track of how much time a window is active."""

    __gsignals__ = {
        'timing-started': (GObject.SignalFlags.RUN_LAST, None, ()),
        'timing-stopped': (GObject.SignalFlags.RUN_LAST, None, ())
        }

    def __init__ (self, window):
        GObject.GObject.__init__(self)
        self.window = window
        # whether we have 'start_timing'; affects total_time
        self.timer_running = False
        # whether the timer is running/paused; affects active_time
        self.is_timing = False

        self.active_time = 0
        self.total_time = 0
        self.__absolute_start_time__ = 0
        self.interval_start = 0

        self.window.connect('focus-in-event',
                lambda *args: self.resume_timing())
        self.window.connect('focus-out-event',
                lambda *args: self.pause_timing())

    def resume_timing (self):
        '''start the interval of active time
        '''
        if self.timer_running and not self.is_timing:
            self.is_timing = True
            self.interval_start = time.time()
            self.emit('timing-started')

    def pause_timing (self):
        '''end the interval of active time
        '''
        if self.timer_running and self.is_timing:
            self.is_timing = False
            interval_end = time.time()
            # active_time is composed of intervals between pausing and resuming
            self.active_time += (interval_end - self.interval_start)
            self.total_time = time.time() - self.__absolute_start_time__
            self.emit('timing-stopped')

    def start_timing (self):
        self.timer_running = True
        self.active_time = 0
        self.total_time = 0
        self.__absolute_start_time__ = time.time()
        self.resume_timing()

    def mark_timing(self):
        currently_timing = self.is_timing
        self.pause_timing()
        if self.active_time < 1:
            self.active_time = 1
        # dirty hack: never let total time be less than active time
        if self.active_time > self.total_time:
            self.total_time = self.active_time
        if currently_timing:
            self.resume_timing()

    def finish_timing (self):
        self.mark_timing()
        self.timer_running = False

if __name__ == '__main__':
    from gi.repository import Gtk

    def report (timer):
        pass

    def test_active_timer ():
        win = Gtk.Window()
        timer = ActiveTimer(win)
        timer.start_timing()
        win.connect('focus-out-event', lambda *args: report(timer))
        win.connect('delete-event', Gtk.main_quit)
        win.show()
        Gtk.main()

    test_active_timer()



# -*- coding: utf-8 -*-
from gi.repository import Gtk,GObject
import colors

class GridDancer:

    DANCE_COLORS = [colors.color_hex_to_float(hx) for hx in
                    [
        '#cc0000', # red
        '#ef2929',
        '#f57900', # orange
        '#fcaf3e',
        '#fce94f',
        '#8ae234', # green
        '#73d216',
        '#729fcf', # blue
        '#3465a4',
        '#ad7fa8', # violet
        '#75507b', ]
                    ]

    STEPS_PER_ANIMATION = 10

    def __init__ (self, grid):
        self.animations = [self.value_dance,
                           self.box_dance,
                           self.col_dance,
                           self.row_dance,]
        self.current_animation = self.value_dance
        self.step = 0
        self.grid = grid
        self.dancing = False
        self.adjustment = 0

    def start_dancing (self):
        for box in self.grid.__entries__.values():
            box.props.can_focus = False
            if box.read_only:
                box.read_only = False
                box.need_restore = True
            else:
                box.need_restore = False
        self.grid.get_toplevel().child_focus(Gtk.DirectionType.TAB_BACKWARD)
        self.dancing = True
        GObject.timeout_add(350, self.dance_grid)

    def stop_dancing (self):
        self.dancing = False
        for box in self.grid.__entries__.values():
            box.props.can_focus = True
            if box.need_restore:
                box.read_only = True
        self.grid.unhighlight_cells()

    def dance_grid (self):
        if not self.dancing:
            return
        if self.step > self.STEPS_PER_ANIMATION:
            self.rotate_animation()
        self.adjustment = (self.adjustment + 1) % 9
        try:
            self.current_animation()
        except AttributeError:
            return True
        self.step += 1
        if self.dancing:
            return True

    def rotate_animation (self):
        current_index = self.animations.index(self.current_animation)
        next_index = (current_index + 1) % len(self.animations)
        self.current_animation = self.animations[next_index]
        self.step = 0

    def next_color (self, current):
        result = (current + self.adjustment) % len(self.DANCE_COLORS)
        return self.DANCE_COLORS[result]

    def col_dance (self):
        for x in range(9):
            color = self.next_color(x)
            for y in range(9):
                self.grid.__entries__[(x, y)].set_background_color(color)

    def row_dance (self):
        for y in range(9):
            color = self.next_color(y)
            for x in range(9):
                self.grid.__entries__[(x, y)].set_background_color(color)

    def box_dance (self):
        for box in range(9):
            color = self.next_color(box)
            for x, y in self.grid.grid.box_coords[box]:
                self.grid.__entries__[(x, y)].set_background_color(color)

    def value_dance (self):
        for value in range(10):
            color = self.next_color(value)
            for x in range(9):
                for y in range(9):
                    box = self.grid.__entries__[(x, y)]
                    if box.get_value() == value:
                        box.set_background_color(color)

if __name__ == '__main__':
    def test_dance_grid ():
        import gsudoku
        window = Gtk.Window()
        game = '''9 1 6 3 2 8 4 5 7
                  5 7 4 6 1 9 2 8 3
                  8 3 2 5 7 4 9 6 1
                  6 8 7 2 4 1 3 9 5
                  2 9 5 7 3 6 1 4 8
                  3 4 1 8 9 5 7 2 6
                  4 6 9 1 8 7 5 3 2
                  1 2 8 9 5 3 6 7 4
                  7 5 3 4 6 2 8 1 9'''
        gsd = gsudoku.SudokuGameDisplay(game)
        dancer = GridDancer(gsd)

        button = Gtk.Button('toggle')
        button.connect('clicked',
                lambda *args: dancer.stop_dancing() if dancer.dancing
                    else dancer.start_dancing())

        vbox = Gtk.VBox()
        vbox.pack_start(gsd, True, True, 0)
        vbox.pack_end(button, True, True, 0)
        vbox.set_focus_child(button)

        window.add(vbox)
        window.show_all()
        window.connect('delete-event', Gtk.main_quit)
        Gtk.main()

    test_dance_grid()



# -*- coding: utf-8 -*-
import time

class MethodWrapper:

    def __call__ (self, cls):
        for attr in dir(cls):
            attrobj = getattr(cls, attr)
            if callable(attrobj) and attr.find('__')!=0:
                setattr(cls, attr, self.wrap(attrobj))

    def wrap (self, f):
        def _(*args, **kwargs):
            self.wrapper(*args, **kwargs)
            return f(*args, **kwargs)
        return _

    def wrapper (self, *args, **kwargs):
        print args, kwargs

class PausableWrapper (MethodWrapper):

    def __init__ (self, sleep_for=1):
        self.sleep_for = sleep_for

    def __call__ (self, cls):
        MethodWrapper.__call__(self, cls)
        cls.paused = False
        cls.terminated = False
        cls.pause = lambda *args: self.pause(cls)
        cls.resume = lambda *args: self.resume(cls)
        cls.terminate = lambda *args: self.terminate(cls)
        cls.stop = lambda *args: self.terminate(cls)
        self.init_wrap(cls.__init__)


    def init_wrap (self, f):
        def _(cls, *args, **kwargs):
            cls.paused = False
            cls.terminated = False
            return f(cls, *args, **kwargs)
        return _

    def pause (self, cls):
        cls.paused = True

    def resume (self, cls):
        cls.paused = False

    def terminate (self, cls):
        cls.terminated = True

    def unterminate (self, cls):
        cls.terminated = False

    def wrapper (self, cls, *args, **kwargs):
        if cls.terminated:
            raise Exception("Terminated!")
        while cls.paused:
            if cls.terminated:
                raise Exception("Terminated!")
            time.sleep(self.sleep_for)


make_pausable = PausableWrapper()



# -*- coding: utf-8 -*-
from gi.repository import Gtk,GObject
import xml.sax.saxutils
from gettext import gettext as _
H_PADDING=12
Y_PADDING=12

class UserCancelledError (Exception):
    pass

class ModalDialog (Gtk.Dialog):
    def __init__ (self, default=None, title="", okay=True, label=False, sublabel=False, parent=None, cancel=True, modal=True, expander=None, image=None):
        """Our basic class. We allow for a label. Possibly an expander
        with extra information, and a simple Okay button.  The
        expander is are only fancy option. It should be a list ['Name
        of expander', CONTENTS]. CONTENTS can be a string (to be put
        in a label), a widget (to be packed in the expander), or a
        list of strings and widgets to be packed in order."""
        self.widget_that_grabs_focus = None
        self.setup_dialog(title=title, parent=parent)
        self.connect('destroy',self.cancelcb)
        self.set_border_width(15)
        self.default = default
        self.ret = default
        self.responses = {Gtk.ResponseType.OK:self.okcb,
                          Gtk.ResponseType.CANCEL:self.cancelcb,
                          Gtk.ResponseType.NONE:self.cancelcb,
                          Gtk.ResponseType.CLOSE:self.cancelcb,
                          Gtk.ResponseType.DELETE_EVENT:self.cancelcb}
        if modal:
            self.set_modal(True)
        if label:
            self.setup_label(label)
        if sublabel:
            self.setup_sublabel(sublabel)
        if expander:
            # if we have an expander, our window
            # should be resizable (just in case
            # the user wants to do more resizing)
            self.set_resizable(True)
            self.setup_expander(expander)
        if cancel or okay:
            self.setup_buttons(cancel, okay)

        self.show_all()

    def setup_dialog (self, *args, **kwargs):
        GObject.GObject.__init__(self, *args, **kwargs)

    def setup_label (self, label):
        # we're going to add pango markup to our
        # label to make it bigger as per GNOME HIG
        label = '<span weight="bold" size="larger">%s</span>'%label
        label = Gtk.Label()
        label.set_line_wrap(True)
        label.set_selectable(True)
        label.set_padding(H_PADDING,Y_PADDING)
        label.set_alignment(0,0)
        label.set_justify(Gtk.Justification.LEFT)
        label.set_markup(label)
        label.show()

        self.get_content_area().pack_start(label,expand=False)
        
    def setup_sublabel (self,sublabel):
        sublabel = Gtk.Label()
        sublabel.set_selectable(True)
        sublabel.set_padding(H_PADDING,Y_PADDING)
        sublabel.set_alignment(0,0)
        sublabel.set_justify(Gtk.Justification.LEFT)
        sublabel.set_markup(sublabel)
        sublabel.set_line_wrap(True)
        sublabel.show()

        self.get_content_area().pack_start(sublabel, False, True, 0)

    def setup_buttons (self, cancel, okay):
        if cancel:
            self.add_button(Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL)
        if okay:
            self.add_button(Gtk.STOCK_OK,Gtk.ResponseType.OK)
        self.connect('response',self.response_cb)

    def response_cb (self, dialog, response, *params):
        if self.responses.has_key(response):
            self.responses[response]()
        else:
            print 'WARNING, no response for ',response
            
    def setup_expander (self, expander):
        label,body = expander
        expander = Gtk.Expander(label)
        expander.set_use_underline(True)
        expander_vbox = Gtk.VBox()
        expander.add(self.expander_vbox)
        self._add_expander_item(expander_vbox, body)
        expander.show_all()

        self.get_content_area().add(expander)
            
    def _add_expander_item (self, expander_vbox, item):
        if type(item) == type(""):
            l = Gtk.Label(label=item)
            l.set_selectable(True)
            l.set_line_wrap(True)
            expander_vbox.pack_start(l, expand=False, fill=False)
        elif type(item) == [] or type(item) == ():
            map(self._add_expander_item, expander_vbox, item)
        else:
            expander_vbox.pack_start(item, True, True, 0)
            
    def run (self):
        self.show()
        if self.widget_that_grabs_focus:
            self.widget_that_grabs_focus.grab_focus()
        if self.props.modal:
            Gtk.main()
        return self.ret

    def okcb (self, *args):
        self.hide()
        if self.props.modal:
            Gtk.main_quit()

    def cancelcb (self, *args):
        self.hide()
        self.ret=None
        if self.props.modal:
            Gtk.main_quit()

class MessageDialog (Gtk.MessageDialog, ModalDialog):

    """A simple class for displaying messages to our users."""
    
    def __init__ (self, title="", default=None, okay=True, cancel=False, label=False, sublabel=False,
                  expander=None, message_type=Gtk.MessageType.INFO, icon=None, parent=None):
        self.message_type=message_type
        self.icon = icon
        ModalDialog.__init__(self, title=title, default=default, okay=okay, cancel=cancel, label=label, sublabel=sublabel, parent=parent, expander=expander)

    def setup_dialog (self, *args, **kwargs):
        kwargs['type']=self.message_type
        if kwargs.has_key('title'):
            del kwargs['title']
        GObject.GObject.__init__(self, *args, **kwargs)
        if self.icon:
            if type(self.icon)==str:
                self.image.set_from_file(self.icon)
            else:
                self.image.set_from_pixbuf(self.icon)
        print "123"

    def setup_label (self, label):
        label = '<span weight="bold" size="larger">%s</span>'%xml.sax.saxutils.escape(label)
        self.set_markup(label)

    def setup_sublabel (self, sublabel):
        self.format_secondary_text(sublabel)


class BooleanDialog (MessageDialog):
    def __init__ (self, title="", default=True, label=_("Do you really want to do this?"),
                  sublabel=False, cancel=True,
                  parent=None, custom_yes=None, custom_no=None, expander=None,
                  dont_ask_cb=None, dont_ask_custom_text=None,
                  cancel_returns=None, message_type=Gtk.MessageType.QUESTION
                  ):
        """Setup a BooleanDialog which returns True or False.
        parent is our parent window.
        custom_yes is custom text for the button that returns true or a dictionary
                   to be handed to Gtk.Button as keyword args.
        custom_no is custom text for the button that returns False or a dictionary
                   to be handed to Gtk.Button as keyword args
        expander is a list whose first item is a label and second is a widget to be packed
        into an expander widget with more information.
        if dont_ask_variable is set, a Don't ask me again check
        button will be displayed which the user can check to avoid this kind
        of question again. (NOTE: if dont_ask_variable==None, this won't work!)
        dont_ask_custom_text is custom don't ask text."""
        self.cancel_returns = cancel_returns
        self.yes,self.no = custom_yes,custom_no        
        if not self.yes: self.yes = Gtk.STOCK_YES
        if not self.no: self.no = Gtk.STOCK_NO
        MessageDialog.__init__(self,title=title,okay=False,label=label, cancel=cancel, sublabel=sublabel,parent=parent, expander=expander, message_type=message_type)
        self.responses[Gtk.ResponseType.YES]=self.yescb
        self.responses[Gtk.ResponseType.NO]=self.nocb
        if not cancel:
            # if there's no cancel, all cancel-like actions
            # are the equivalent of a NO response
            self.responses[Gtk.ResponseType.NONE]=self.nocb
            self.responses[Gtk.ResponseType.CANCEL]=self.nocb
            self.responses[Gtk.ResponseType.CLOSE]=self.nocb
            self.responses[Gtk.ResponseType.DELETE_EVENT]=self.nocb
        if dont_ask_cb:
            if not dont_ask_custom_text:
                dont_ask_custom_text=_("Don't ask me this again.")
            self.dont_ask = Gtk.CheckButton(dont_ask_custom_text)
            self.dont_ask.connect('toggled',dont_ask_cb)
            self.get_content_area().add(self.dont_ask)
            self.dont_ask.show()

    def setup_buttons (self, cancel, okay):
        MessageDialog.setup_buttons(self,cancel,None)
        self.add_button(self.no,Gtk.ResponseType.NO)
        self.add_button(self.yes,Gtk.ResponseType.YES)

    def yescb (self, *args):
        self.ret=True
        self.okcb()

    def cancelcb (self, *args):
        if self.cancel_returns != None:
            self.ret = self.cancel_returns
            self.okcb()
        else:
            self.hide()
            if self.props.modal:
                Gtk.main_quit()

    def nocb (self, *args):
        self.ret=False
        self.okcb()

def show_message_dialog (*args, **kwargs):
    d = MessageDialog(*args, **kwargs)
    return d.run()

def show_boolean_dialog (*args,**kwargs):
    """Run BooleanDialog, passing along all args, waiting on input and
    passing along the results."""
    d = BooleanDialog(*args,**kwargs)
    retval = d.run()
    if retval == None:
        raise UserCancelledError("show_boolean_dialog dialog cancelled!")
    else:
        return retval



# -*- coding: utf-8 -*-
from gi.repository import Gtk
import pickle, types, os, errno
import defaults
from gtk_goodies.dialog_extras import show_message_dialog
from gettext import gettext as _
import tracker_info

SAVE_ATTRIBUTES = [('gsd.hints'),
                   ('gsd.impossible_hints'),
                   ('timer.__absolute_start_time__'),
                   ('timer.active_time'),
                   ('timer.total_time'),
                   ]

def update_saved_attributes (jar):
    '''Make sure all SAVE_ATTRIBUTES are available and valid.
    '''
    for attr in SAVE_ATTRIBUTES:
        # default to 0, which is reasonable for current SAVE_ATTRIBUTES.
        if not attr in jar:
            jar[attr] = 0
    # special case for timing things
    if 'timer.tot_time' in jar: # tot_time was renamed to active_time
        jar['timer.active_time'] = jar['timer.tot_time']
    if 'timer.tot_time_complete' in jar: # tot_time_complete renamed to total_time
        jar['timer.total_time'] = jar['timer.tot_time_complete']
    if not jar['timer.active_time']:
        # FIXME set to 1 in order to display well at game-selecting page
        jar['timer.active_time'] = 1

def super_getattr (obj, attr):
    """getattr, following the dots."""
    attrs = attr.split('.')
    for a in attrs:
        obj = getattr(obj, a)
    return obj

def super_setattr (obj, attr, val):
    """setattr, following the dots."""
    attrs = attr.split('.')
    if len(attrs) > 1:
        sub_attrs = attrs[0:-1]
        attr = attrs[-1]
        for a in sub_attrs:
            obj = getattr(obj, a)
    setattr(obj, attr, val)

def jar_game (ui):
    jar = {} # what we will pickle
    ui.timer.mark_timing()
    jar['game'] = ui.gsd.grid.to_string()
    jar['tracker_info'] = tracker_info.TrackerInfo().save()
    jar['tracked_notes'] = []
    for e in ui.gsd.__entries__.values():
        if e.top_note_list or e.bottom_note_list:
            jar['tracked_notes'].append((e.x, e.y, e.top_note_list, e.bottom_note_list))
    for attr in SAVE_ATTRIBUTES:
        jar[attr] = super_getattr(ui, attr)
    return jar

def set_value_from_jar (dest, jar):
    for attr in SAVE_ATTRIBUTES:
        super_setattr(dest, attr, jar[attr])

def open_game (ui, jar):
    tinfo = tracker_info.TrackerInfo()
    tinfo.set_tracker(tracker_info.NO_TRACKER)
    ui.gsd.load_game(jar['game'])
    # The 'notes' and 'trackers' sections are for transition from the old
    # style tracker storage.  The tracker values and notes are stored in the
    # 'tracked_notes' and 'tracker_info' sections now.
    if jar.has_key('notes') and jar['notes']:
        for x, y, top, bot in jar['notes']:
            ui.gsd.__entries__[(x, y)].set_note_text(top, bot)
    if jar.has_key('trackers'):
        for tracker, tracked in jar.get('trackers', {}).items():
            # add 1 tracker per existing tracker...
            ui.tracker_ui.add_tracker()
            for x, y, val in tracked:
                tinfo.add_trace(x, y, val)
    set_value_from_jar(ui, jar)
    if jar.has_key('tracking'):
        for tracker, tracking in jar.get('tracking', {}).items():
            if tracking:
                ui.tracker_ui.select_tracker(tracker)
    if jar.has_key('tracked_notes') and jar['tracked_notes']:
        for x, y, top, bot in jar['tracked_notes']:
            ui.gsd.__entries__[(x, y)].set_notelist(top, bot)
    if jar.has_key('tracker_info'):
        trackers = jar['tracker_info'][2]
        for tracking in trackers.keys():
            ui.tracker_ui.add_tracker(tracker_id = tracking)
        tinfo.load(jar['tracker_info'])
        ui.tracker_ui.select_tracker(tinfo.current_tracker)
        if tinfo.showing_tracker != tracker_info.NO_TRACKER:
            ui.gsd.show_track()
    # Display the notes
    ui.gsd.update_all_notes()

def pickle_game (ui, target):
    close_me = False
    if type(target) in types.StringTypes:
        target = file(target, 'w')
        close_me = True
    to_dump = jar_game(ui)
    pickle.dump(to_dump, target)
    if close_me:
        target.close()

def unpickle_game (ui, target):
    close_me = False
    if type(target) == str:
        target = file(target, 'r')
        close_me = True
    open_game(ui, pickle.load(target))
    if close_me:
        target.close()

class SudokuTracker:

    """A class to track games in progress and games completed.
    """

    def __init__ (self):
        self.save_path = os.path.join(defaults.DATA_DIR,"saved")
        self.finished_path = os.path.join(defaults.DATA_DIR,"finished")
        self.migrate_data()
        self.create_dir_safely(self.save_path)
        self.create_dir_safely(self.finished_path)

    def migrate_data (self):
        old_path = os.path.expanduser('~/.sudoku')
        if os.path.exists(old_path) and (not os.path.exists(self.save_path) and not os.path.exists(self.finished_path)):
            try:
                os.rename(os.path.join(old_path,"saved"),self.save_path)
            except:
                pass
            try:
                os.rename(os.path.join(old_path,"finished"),self.finished_path)
            except:
                pass
            try:
                os.rmdir(old_path)
            except:
                pass

    def create_dir_safely (self, path):
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except OSError, e:
                if e.errno == errno.ENOSPC:
                    show_message_dialog(
                        title = _('No Space'),
                        label = _('No space left on disk'),
                        message_type = Gtk.MessageType.ERROR,
                        sublabel = _('Unable to create data folder %(path)s.') % locals() + '\n' + \
                                   _('There is no disk space left!')
                        )
                else:
                    show_message_dialog(
                        title = 'Error creating directory',
                        label = 'Error creating directory',
                        sublabel = (_('Unable to create data folder %(path)s.') % locals() + '\n' +
                                    _('Error %(errno)s: %(error)s') % {
                                        'errno': e.errno,
                                        'error': e.strerror})
                        )

    def game_from_ui (self, ui):
        return ui.gsd.grid.virgin.to_string()

    def get_filename (self, gamestring):
        return gamestring.split('\n')[0].replace(' ', '')

    def save_game (self, ui):
        jar = jar_game(ui)
        filename = os.path.join(self.save_path, self.get_filename(jar['game']))
        try:
            outfi = file(filename, 'w')
            pickle.dump(jar, outfi)
            outfi.close()
        except (OSError, IOError), e:
            show_message_dialog(
                title = _('Unable to save game.'),
                label = _('Unable to save game.'),
                message_type = Gtk.MessageType.ERROR,
                sublabel = (_('Unable to save file %(filename)s.') % locals()
                          + '\n' +
                          _('Error %(errno)s: %(error)s') % {
                'errno':e.errno,
                'error':e.strerror
                })
                )

    def finish_game (self, ui):
        jar  = jar_game(ui)
        self.finish_jar(jar)

    def finish_jar (self, jar):
        self.remove_from_saved_games(jar)
        try:
            filename = os.path.join(self.finished_path,
                                    self.get_filename(jar['game']))
            outfi = file(filename, 'w')
            pickle.dump(jar, outfi)
            outfi.close()
        except (OSError, IOError), e:
            show_message_dialog(
                title = _('Unable to mark game as finished.'),
                label = _('Unable to mark game as finished.'),
                message_type = Gtk.MessageType.ERROR,
                sublabel = (_('Unable to save file %(filename)s.') % locals() + '\n' +
                          _('Error %(errno)s: %(error)s') % {
                'errno':e.errno,
                'error':e.strerror
                })
                )
        try:
            filename = list_of_finished_games = os.path.join(
                os.path.join(defaults.DATA_DIR, 'puzzles'), 'finished'
                )
            ofi = open(list_of_finished_games, 'a')
            ofi.write(jar['game'].split('\n')[0]+'\n')
            ofi.close()
        except (OSError, IOError), e:
            show_message_dialog(
                title = _('Sudoku unable to mark game as finished.'),
                label = _('Sudoku unable to mark game as finished.'),
                message_type = Gtk.MessageType.ERROR,
                sublabel = (_('Unable to save file %(filename)s.') % locals() + '\n' +
                          _('Error %(errno)s: %(error)s') % {
                'errno':e.errno,
                'error':e.strerror
                })
                )

    def remove_from_saved_games (self, jar):
        previously_saved_game = os.path.join(
            self.save_path, self.get_filename(jar['game'])
            )
        if os.path.exists(previously_saved_game):
            os.remove(os.path.join(previously_saved_game))

    def abandon_game (self, ui):
        jar  = jar_game(ui)
        self.remove_from_saved_games(jar)

    def list_saved_games (self):
        try:
            files = os.listdir(self.save_path)
        except OSError:
            files = []
        games = []
        for f in files:
            f = os.path.join(self.save_path, f)
            try:
                jar = pickle.load(file(f, 'r'))
            except:
                print 'Warning: could not read file', f
            else:
                update_saved_attributes(jar)
                if self.is_valid(jar):
                    jar['saved_at'] = os.stat(f)[8]
                    games.append(jar)
                else:
                    print 'Warning: malformed save game', f
        return games

    def is_valid (self, jar):
        virgin = jar['game'].split('\n')[0].replace(' ', '')
        played = jar['game'].split('\n')[1].replace(' ', '')

        if len(virgin) != 81 or len(played) != 81:
            return False

        if not virgin.isdigit() or not played.isdigit():
            return False

        for attr in SAVE_ATTRIBUTES:
            if jar.get(attr, None) == None:
                return False

        return True



# -*- coding: utf-8 -*-
import random
import math
import re
from gettext import gettext as _
import defaults

GROUP_SIZE = 9

TYPE_ROW = 0
TYPE_COLUMN = 1
TYPE_BOX = 2

digit_set = range(1, GROUP_SIZE + 1)
sets = [digit_set] * 9

def is_set (row):
    if len(row) == len(set(row)):
        return True

def is_sudoku (rows):
    # check rows
    for r in rows:
        if not is_set(r):
            return False
    for i in range(len(rows[0])):
        rw = [r[i] for r in rows]
        if not is_set(rw):
            return False
    # check boxes
    width = int(math.sqrt(len(rows)))
    # there should be 3x3 boxes, or 4x4 if we got funky, etc.
    # boxes will be indices
    box_coordinates = [[n * width,
                        (n + 1) * width] for n in range(width)]
    for x in box_coordinates:
        for y in box_coordinates:
            box = []
            for xrow in [rows[ri] for ri in range(*y)]:
                for i in range(*x):
                    box.append(xrow[i])
            if not is_set(box):
                return False
    return True

class UnsolvablePuzzle (TypeError):
    pass


class ConflictError (ValueError):

    def __init__ (self, conflict_type, coordinates, value):
        self.args = conflict_type, coordinates, value
        self.type = conflict_type
        self.coordinates = coordinates
        self.x = coordinates[0]
        self.y = coordinates[1]
        self.value = value

class AlreadySetError (ValueError):
    pass

class ParallelDict (dict):
    """A handy new sort of dictionary for tracking conflicts.

    pd = ParallelDict()
    pd[1] = [2, 3, 4] # 1 is linked with 2, 3 and 4
    pd -> {1:[2, 3, 4], 2:[1], 3:[1], 4:[1]}
    pd[2] = [1, 3, 4] # 2 is linked with 3 and 4 as well as 1
    pd -> {1: [2, 3, 4], 2:[3, 4], 3:[1, 2], 4:[1, 2]}
    Now for the cool part...
    del pd[1]
    pd -> {2: [2, 3], 3:[2], 4:[2]}

    Pretty neat, no?
    """
    def __init__ (self, *args):
        dict.__init__(self, *args)

    def __setitem__ (self, k, v):
        dict.__setitem__(self, k, set(v))
        for i in v:
            if i == k:
                continue
            if self.has_key(i):
                self[i].add(k)
            else:
                dict.__setitem__(self, i, set([k]))

    def __delitem__ (self, k):
        v = self[k]
        dict.__delitem__(self, k)
        for i in v:
            if i == k:
                continue
            if self.has_key(i):
                # Make sure we have a reference to i. If we don't
                # something has gone wrong... but according to bug
                # 385937 this has gone wrong at least once, so we'd
                # better check for it.
                if k in self[i]:
                    self[i].remove(k)
                if not self[i]:
                    # If k was the last value in the list of values
                    # for i, then we delete i from our dictionary
                    dict.__delitem__(self, i)

class SudokuGrid(object):
    def __init__ (self, grid = False, verbose = False, group_size = 9):
        self.grid = []
        self.cols = []
        self.rows = []
        self.boxes = []
        self.conflicts = ParallelDict()
        self.group_size = int(group_size)
        self.verbose = False
        self.gen_set = set(range(1, self.group_size + 1))
        for n in range(self.group_size):
            self.cols.append(set())
            self.rows.append(set())
            self.boxes.append(set())
            self.grid.append([0] * self.group_size)
        self.box_by_coords = {}
        self.box_coords = {}
        self.calculate_box_coords() # sets box_coords and box_by_coords
        self.row_coords = {}
        for n, row in enumerate([[(x, y) for x in range(self.group_size)] for y in range(self.group_size)]):
            self.row_coords[n] = row
        self.col_coords = {}
        for n, col in enumerate([[(x, y) for y in range(self.group_size)] for x in range(self.group_size)]):
            self.col_coords[n] = col
        if grid:
            if isinstance(grid, basestring):
                g = re.split("\s+", grid)
                side = int(math.sqrt(len(g)))
                grid = []
                for row in range(side):
                    start = row * int(side)
                    grid.append([int(i) for i in g[start:start + side]])
            self.populate_from_grid(grid)
        self.verbose = verbose

    def calculate_box_coords (self):
        width = int(math.sqrt(self.group_size))
        box_coordinates = [[n * width,
                            (n + 1) * width] for n in range(width)]
        box_num = 0
        for xx in box_coordinates:
            for yy in box_coordinates:
                self.box_coords[box_num] = []
                for x in range(*xx):
                    for y in range(*yy):
                        self.box_by_coords[(x, y)] = box_num
                        self.box_coords[box_num].append((x, y))
                box_num += 1

    def add (self, x, y, val, force = False):
        if not val:
            pass
        if self._get_(x, y):
            if force:
                try:
                    self.remove(x, y)
                except:
                    print 'Strange: problem with add(', x, y, val, force, ')'
                    import traceback
                    traceback.print_exc()
            else:
                #FIXME:  This is called when the fill button
                #is clicked multiple times, which causes this exception:
                #raise AlreadySetError
                return
        # Always store the value in the underlying grid
        self._set_(x, y, val)
        # But don't add it to the solution hints(rows/cols/boxes) if there is
        # a conflict
        if val in self.rows[y]:
            raise ConflictError(TYPE_ROW, (x, y), val)
        if val in self.cols[x]:
            raise ConflictError(TYPE_COLUMN, (x, y), val)
        box = self.box_by_coords[(x, y)]
        if val in self.boxes[box]:
            raise ConflictError(TYPE_BOX, (x, y), val)
        # do the actual adding
        self.rows[y].add(val)
        self.cols[x].add(val)
        self.boxes[box].add(val)

    def remove (self, x, y):
        val = self._get_(x, y)
        self.rows[y].discard(val)
        self.cols[x].discard(val)
        self.boxes[self.box_by_coords[(x, y)]].discard(val)
        self._set_(x, y, 0)

    def _get_ (self, x, y):
        return self.grid[y][x]

    def _set_ (self, x, y, val):
        self.grid[y][x] = val

    def possible_values (self, x, y):
        return self.gen_set - self.rows[y] - self.cols[x] - self.boxes[self.box_by_coords[(x, y)]]

    def pretty_print (self):
        print 'SUDOKU'
        for r in self.grid:
            for i in r:
                print i,
            print
        print

    def populate_from_grid (self, grid):
        for y, row in enumerate(grid):
            for x, cell in enumerate(row):
                if cell:
                    try:
                        self.add(x, y, cell)
                    except ConflictError:
                        pass

    def __repr__ (self):
        s = "<Grid\n       "
        grid = []
        for r in self.grid:
            grid.append(" ".join([str(i) for i in r]))
        s += "\n       ".join(grid)
        return s

    def calculate_open_squares (self):
        possibilities = {}
        for x in range(self.group_size):
            for y in range(self.group_size):
                if not self._get_(x, y):
                    possibilities[(x, y)] = self.possible_values(x, y)
        return possibilities

    def to_string (self):
        """Output our grid as a string."""
        return " ".join([" ".join([str(x) for x in row]) for row in self.grid])

def is_valid_puzzle (p):
    """Check puzzle for basic validity.

    This does not check for solvability or ensure a unique
    solution -- it merely checks well-formedness. This should
    provide some protection again file corruption, etc. (i.e. if
    we use this function to check puzzles before handing them out
    to the rest of the app, we'll prevent tracebacks related to
    corrupted puzzles).
    """
    try:
        p = p.replace(' ', '')
        assert(len(p.replace(' ', '')) == 81)
        [int(c) for c in p.replace(' ', '')]
    except:
        #import traceback; traceback.print_exc()
        return False
    else:
        return True

def sudoku_grid_from_string (s):
    """Given an 81 character string, return a grid."""
    s = s.replace(' ', '')
    assert(len(s)<=GROUP_SIZE ** 2)
    grid = []
    i = 0
    for x in range(GROUP_SIZE):
        row = []
        for y in range(GROUP_SIZE):
            if len(s) <= i:
                n = 0
            else:
                n = s[i]
            try:
                n = int(n)
            except:
                n = n or 0
            if n in digit_set:
                row.append(n)
            else:
                row.append(0)
            i += 1
        grid.append(row)
    return SudokuGrid(grid)


class SudokuSolver (SudokuGrid):
    """A SudokuGrid that can solve itself."""
    def __init__ (self, grid = False, verbose = False, group_size = 9):
        self.current_guess = None
        self.initialized = False
        SudokuGrid.__init__(self, grid, verbose = verbose, group_size = group_size)
        self.virgin = SudokuGrid(grid)
        self.guesses = GuessList()
        self.breadcrumbs = BreadcrumbTrail()
        self.backtraces = 0
        self.initialized = True
        self.solved = False
        self.solving = False
        self.trail = []

    def auto_fill_for_xy (self, x, y):
        """Fill the square x,y if possible."""
        possible = self.gen_set - self.rows[y] - self.cols[x] - self.boxes[self.box_by_coords[(x, y)]]
        if len(possible) == 1:
            val = possible.pop()
            self.add(x, y, val)
            return ((x, y), val)
        if len(possible) == 0:
            return -1
        # check our column...
        for coord_set, filled_set in [(self.col_coords[x], self.cols[x]),
                                     (self.row_coords[y], self.rows[y]),
                                     (self.box_coords[self.box_by_coords[(x, y)]],
                                      self.boxes[self.box_by_coords[(x, y)]])
                                     ]:
            needed_set = self.gen_set - filled_set
            for coord in coord_set:
                if self._get_(*coord):
                    continue
                elif (x, y) != coord:
                    needed_set = needed_set - self.possible_values(*coord)
            if needed_set and len(needed_set) == 1:
                val = needed_set.pop()
                if val in possible:
                    self.add(x, y, val)
                    return ((x, y), val)
                else:
                    return -1
            if len(needed_set)>1:
                return -1

    def auto_fill (self):
        changed = []
        try:
            changed = self.fill_must_fills()
        except UnsolvablePuzzle:
            return changed
        try:
            changed.extend(self.fill_deterministically())
        finally:
            return changed

    def fill_must_fills (self):
        changed = []
        for label, coord_dic, filled_dic in [('Column', self.col_coords, self.cols),
                                           ('Row', self.row_coords, self.rows),
                                           ('Box', self.box_coords, self.boxes)]:
            for n, coord_set in coord_dic.items():
                skip_set = False
                for coord in coord_set:
                    if self.conflicts.has_key(coord):
                        skip_set = True
                        break
                if skip_set:
                    continue
                needs = dict([(n, False) for n in range(1, self.group_size + 1)])
                for coord in coord_set:
                    val = self._get_(*coord)
                    if val:
                        # We already have this value set...
                        if needs.has_key(val):
                            del needs[val]
                    else:
                        # Otherwise, register ourselves as possible
                        # for each number we could be
                        for v in self.possible_values(*coord):
                            # if we don't yet have a possible number, plug ourselves in
                            if needs.has_key(v):
                                if not needs[v]:
                                    needs[v] = coord
                                else:
                                    del needs[v]
                for n, coords in needs.items():
                    if not coords:
                        raise UnsolvablePuzzle('Missing a %s in %s' % (n, label))
                    else:
                        try:
                            self.add(coords[0], coords[1], n)
                            changed.append((coords, n))
                        except AlreadySetError:
                            raise UnsolvablePuzzle(
                                "%s,%s must be two values at once!" % (coords)
                                )
        return changed

    def fill_deterministically (self):
        poss = self.calculate_open_squares().items()
        one_choice = filter(lambda x: len(x[1]) == 1, poss)
        retval = []
        for coords, choices in one_choice:
            if self.verbose:
                print 'Deterministically adding ', coords, choices
            val = choices.pop()
            self.add(coords[0], coords[1], val)
            retval.append([(coords[0], coords[1]), val])
        if self.verbose:
            print 'deterministically returning ', retval
        return retval

    def solve (self):
        if self.solving:
            return
        self.solving = True
        self.auto_fill()
        while not self.guess_least_open_square():
            pass
        if self.verbose:
            print 'Solved!\n', self
        self.solving = False
        self.solved = True

    def solution_finder (self):
        self.auto_fill()
        while not self.guess_least_open_square():
            pass
        self.solved = True
        yield tuple([tuple(r) for r in self.grid[0:]])
        while self.breadcrumbs:
            self.unwrap_guess(self.breadcrumbs[-1])
            try:
                while not self.guess_least_open_square():
                    pass
            except UnsolvablePuzzle:
                break
            else:
                yield tuple([tuple(r) for r in self.grid[0:]])
        yield None

    def has_unique_solution (self):
        sf = self.solution_finder()
        sf.next()
        if sf.next():
            return False
        else:
            return True

    def guess_least_open_square (self):
        # get open squares and check them
        poss = self.calculate_open_squares().items()
        # if there are no open squares, we're done!
        if not poss:
            if self.verbose:
                print 'Solved!'
            return True
        # otherwise, find the possibility with the least possibilities
        poss.sort(lambda a, b: len(a[1]) > len(b[1]) and 1 or len(a[1]) < len(b[1]) and -1 or \
                  a[0] > b[0] and 1 or a[1] < b[1] and -1 or 0)
        least = poss[0]
        # remove anything we've already guessed
        possible_values = least[1] - self.guesses.guesses_for_coord(*least[0])
        if not possible_values:
            if self.breadcrumbs:
                self.backtraces += 1
                self.unwrap_guess(self.breadcrumbs[-1])
                return self.guess_least_open_square()
            else:
                raise UnsolvablePuzzle("Unsolvable %s.\n \
                Out of guesses for %s. Already guessed\n \
                %s (other guesses are %s)" % (self,
                                            least[0],
                                            self.guesses.guesses_for_coord(*least[0]),
                                            self.guesses))
        guess = random.choice(list(possible_values))
        # Create guess object
        guess_obj = Guess(least[0][0], least[0][1], guess)
        if self.breadcrumbs:
            self.breadcrumbs[-1].children.append(guess_obj)
        self.current_guess = None #reset (we're tracked via guess.get_child())
        self.add(least[0][0], least[0][1], guess)
        self.current_guess = guess_obj # (All deterministic additions
                                       # get added to our
                                       # consequences)
        self.guesses.append(guess_obj)
        self.trail.append(('+', guess_obj))
        self.breadcrumbs.append(guess_obj)
        try:
            self.auto_fill()
        except NotImplementedError:
            self.trail.append('Problem filling coordinates after guess')
            self.unwrap_guess(guess_obj)
            return self.guess_least_open_square()
        if set([]) in self.calculate_open_squares().values():
            self.trail.append('Guess leaves us with impossible squares.')
            self.unwrap_guess(guess_obj)
            return self.guess_least_open_square()

    def unwrap_guess (self, guess):
        self.trail.append(('-', guess))
        if self._get_(guess.x, guess.y):
            self.remove(guess.x, guess.y)
        for consequence in guess.consequences.keys():
            if self._get_(*consequence):
                self.remove(*consequence)
        for child in guess.children:
            self.unwrap_guess(child)
            if child in self.guesses:
                self.guesses.remove(child)
        if guess in self.breadcrumbs:
            self.breadcrumbs.remove(guess)

    def pad (self, n, pad_to):
        n = str(n)
        padding = int(pad_to) - len(n)
        second_half = padding / 2
        first_half = second_half + padding % 2
        return " " * first_half + n + " " * second_half

    def add (self, x, y, val, *args, **kwargs):
        if self.current_guess:
            self.current_guess.add_consequence(x, y, val)
        SudokuGrid.add(self, x, y, val, *args, **kwargs)


class InteractiveSudoku (SudokuSolver):
    """A subclass of SudokuSolver that provides some convenience
    functions for helping along a human.who is in the midst of
    solving."""
    def __init__ (self, grid = False, verbose = False, group_size = 9):
        SudokuSolver.__init__(self, grid, verbose, group_size)
        self.cleared_conflicts = []

    def to_string (self):
        return self.virgin.to_string() + '\n' + SudokuSolver.to_string(self)

    def find_impossible_implications (self, x, y):
        """Return a set of impossibilities implied by the users actions."""
        row_cells = self.row_coords[y]
        col_cells = self.col_coords[x]
        box = self.box_by_coords[(x, y)]
        box_cells = self.box_coords[box]
        broken = set()
        for coord_set in [row_cells, col_cells, box_cells]:
            # just work on the open squares
            coord_set = filter(lambda coords: not self._get_(*coords), coord_set)
            for coords in coord_set:
                if not self.possible_values(*coords):
                    broken.add(coords)
        return broken

    def check_for_completeness (self):
        for r in self.rows:
            if len(r) != self.group_size:
                return False
        for c in self.cols:
            if len(c) != self.group_size:
                return False
        return True

    def is_changed (self):
        return (self.grid != self.virgin.grid)

    def add (self, x, y, val, force = False):
        '''Add a value to the grid.

        The main feature of this method is conflict resolution.  When conflicts
        are found they are stored in the conflicts ParallelDict.  A cell that
        is in conflict is stored in the underlying grid(SudokuGrid.grid), but
        it has all of its solution hints cleared(SudokuGrid.rows/cols/boxes).
        Care must be taken so that solution hints from the original
        grid(SudokuSolver.virgin) are not cleared.
        '''
        # First just add it to SudokuGrid
        no_exception = True
        try:
            super(InteractiveSudoku, self).add(x, y, val, force)
        except ConflictError:
            no_exception = False

        # Find any cells that conflict with the new value for this cell
        coords = set([])
        coords.update(self.row_coords[y])
        coords.update(self.col_coords[x])
        coords.update(self.box_coords[self.box_by_coords[(x, y)]])
        coords.discard((x, y))
        conflicting_coordinates = []
        for xx, yy in coords:
            if self._get_(xx, yy) == val:
                conflicting_coordinates.append((xx, yy))
        # Store the conflicts for access
        if conflicting_coordinates:
            self.conflicts[(x, y)] = conflicting_coordinates
        # Resume when there are no conflicts
        else:
            return
        # But when we do have conflicts, the values from cols/rows/boxes need
        # to be removed so the hinting doesn't consider them. We must be
        # chaste with the virgin though.
        try:
            if no_exception and not self.virgin._get_(x, y):
                self.rows[y].discard(val)
                self.cols[x].discard(val)
                self.boxes[self.box_by_coords[(x, y)]].discard(val)
            for xx, yy in conflicting_coordinates:
                if self.virgin._get_(xx, yy):
                    continue
                if not val in self.virgin.rows[yy]:
                    self.rows[yy].discard(val)
                if not val in self.virgin.cols[xx]:
                    self.cols[xx].discard(val)
                if not val in self.virgin.box_coords[self.box_by_coords[(xx, yy)]]:
                    self.boxes[self.box_by_coords[(xx, yy)]].discard(val)
        # This class can be used before the virgin is created.  Pass through
        # for the initialization phase
        except AttributeError:
            pass

    def remove (self, x, y):
        '''Remove a value from the grid.

        The main feature of this method is conflict resolution.  All
        conflicting cells are checked to see if they are actually
        conflict-free.  A list of conflict-free cells are stored in
        InteractiveSudoku.cleared_conflicts.  The cleared_conflicts list is
        cleared for each meaningful call to remove(), so it must be processed
        before another remove() call.
        All solution hints(SudokuGrid.rows/cols/boxes) are reinstated for
        conflict-free cells.
        '''
        # Grab the value that we're clearing.  Skip out if its nothing
        val = self._get_(x, y)
        if not val:
            return
        # Pop the conflicts resolved by this removal
        self.cleared_conflicts = []
        errors_removed = []
        if self.conflicts.has_key((x, y)):
            errors_removed = self.conflicts[(x, y)]
            del self.conflicts[(x, y)]
        # If there are no conflicts for this cell then just remove it in from
        # the grid
        else :
            super(InteractiveSudoku, self).remove(x, y)
            return
        # Grid clearance flags
        if val in self.rows[y]:
            clear_row = True
        else:
            clear_row = False
        if val in self.cols[x]:
            clear_col = True
        else:
            clear_col = False
        if val in self.boxes[self.box_by_coords[(x, y)]]:
            clear_box = True
        else:
            clear_box = False
        # Scroll through the conflicts
        for coord in errors_removed:
            # If it is not an error by some other pairing, append it to a list
            # of conflicts that were actually cleared by this removal.
            if not self.conflicts.has_key(coord):
                self.cleared_conflicts.append(coord)
            # When a conflict remains, we need to correct the rows, cols, and
            # boxes arrays properly
            else:
                if clear_row and coord in self.row_coords[y]:
                    clear_row = False
                if clear_col and coord in self.col_coords[x]:
                    clear_col = False
                if clear_box and coord in self.box_coords[self.box_by_coords[(x, y)]]:
                    clear_box = False

        # Clear the rows, cols, and boxes if we need to.
        if clear_row:
            self.rows[y].remove(val)
        if clear_col:
            self.cols[x].remove(val)
        if clear_box:
            self.boxes[self.box_by_coords[(x, y)]].remove(val)
        # Clear the cell
        self._set_(x, y, 0)

        # Scroll through the cleared conflicts and commit them to ensure they
        # are represented in the grid properly.  It is possible for add() to do
        # subsequent remove()s, so hold onto the cleared conflict list for the
        # caller.
        hold_conflicts = self.cleared_conflicts
        for xx, yy in self.cleared_conflicts:
            self.add(xx, yy, val, True)
        self.cleared_conflicts = hold_conflicts


class DifficultyRating:
    very_hard_range = (0.75, 10)
    hard_range = (0.6, 0.75)
    medium_range = (0.45, 0.6)
    easy_range = (-10, 0.45)

    categories = {'very hard':very_hard_range,
                  'hard':hard_range,
                  'medium':medium_range,
                  'easy':easy_range}

    ordered_categories = ['easy', 'medium', 'hard', 'very hard']

    def __init__ (self,
                  fill_must_fillables,
                  elimination_fillables,
                  guesses,
                  backtraces,
                  squares_filled):
        self.fill_must_fillables = fill_must_fillables
        self.elimination_fillables = elimination_fillables
        self.guesses = guesses
        self.backtraces = backtraces
        self.squares_filled = squares_filled
        if self.fill_must_fillables:
            self.instant_fill_fillable = float(len(self.fill_must_fillables[0]))
        else:
            self.instant_fill_fillable = 0.0
        if self.elimination_fillables:
            self.instant_elimination_fillable = float(len(self.elimination_fillables[0]))
        else:
            self.instant_elimination_fillable = 0.0

        self.proportion_instant_elimination_fillable = self.instant_elimination_fillable / self.squares_filled
        # some more numbers that may be crazy...
        self.proportion_instant_fill_fillable = self.instant_fill_fillable / self.squares_filled
        self.elimination_ease = add_with_diminishing_importance(
            self.count_values(self.elimination_fillables)
            )
        self.fillable_ease = add_with_diminishing_importance(
            self.count_values(self.fill_must_fillables)
            )
        self.value = self.calculate()

    def count_values (self, dct):
        kk = dct.keys()
        kk.sort()
        return [len(dct[k]) for k in kk]

    def calculate (self):
        return 1 - float(self.fillable_ease) / self.squares_filled \
                 - float(self.elimination_ease) / self.squares_filled \
                 + len(self.guesses) / self.squares_filled \
                 + self.backtraces / self.squares_filled

    def __repr__ (self):
        return '<DifficultyRating %s>' % self.value

    def pretty_print (self):
        for name, stat in [('Number of moves instantly fillable by elimination',
                           self.instant_elimination_fillable),
                          ('Percentage of moves instantly fillable by elimination',
                           self.proportion_instant_elimination_fillable * 100),
                          ('Number of moves instantly fillable by filling',
                           self.instant_fill_fillable),
                          ('Percentage of moves instantly fillable by filling',
                           self.proportion_instant_fill_fillable * 100),
                          ('Number of guesses made',
                           len(self.guesses)),
                          ('Number of backtraces', self.backtraces),
                          ('Ease by filling', self.fillable_ease),
                          ('Ease by elimination', self.elimination_ease),
                          ('Calculated difficulty', self.value)
                          ]:
            print name, ': ', stat

    def value_category (self):
        """Get category string, without i18n or capitalization

        For use in categorizing category.
        """
        if self.value > self.very_hard_range[0]:
            return 'very hard'
        elif self.value > self.hard_range[0]:
            return 'hard'
        elif self.value > self.medium_range[0]:
            return 'medium'
        else:
            return 'easy'

def get_difficulty_category (diff_float):
    for category, range in DifficultyRating.categories.items():
        if range[0] <= diff_float < range[1]:
            return category

class SudokuRater (SudokuSolver):

    def __init__ (self, grid = False, verbose = False, group_size = 9):
        self.initialized = False
        self.guessing = False
        self.fake_add = False
        self.fake_additions = []
        self.filled = set([])
        self.fill_must_fillables = {}
        self.elimination_fillables = {}
        self.tier = 0
        SudokuSolver.__init__(self, grid, verbose, group_size)

    def add (self, *args, **kwargs):
        if not self.fake_add:
            if self.initialized and not self.guessing:
                self.scan_fillables()
                for delayed_args in self.add_me_queue:
                    coords = (delayed_args[0], delayed_args[1])
                    if not self._get_(*coords):
                        SudokuSolver.add(self, *delayed_args)
                if not self._get_(args[0], args[1]):
                    SudokuSolver.add(self, *args)
                self.tier += 1
            else:
                SudokuSolver.add(self, *args, **kwargs)
        else:
            self.fake_additions.append(args)

    def scan_fillables (self):
        self.fake_add = True
        # this will now tell us how many squares at current
        # difficulty could be filled at this moment.
        self.fake_additions = []
        try:
            self.fill_must_fills()
        except:
            pass
        self.fill_must_fillables[self.tier] = set(self.fake_additions[:]) - self.filled
        self.add_me_queue = self.fake_additions[:]
        self.fake_additions = []
        try:
            self.fill_deterministically()
        except:
            pass
        self.elimination_fillables[self.tier] = set(self.fake_additions[:]) - self.filled
        self.filled = self.filled | self.fill_must_fillables[self.tier] | self.elimination_fillables[self.tier]
        self.add_me_queue.extend(self.fake_additions[:])
        self.fake_add = False

    def guess_least_open_square (self):
        self.guessing = True
        return SudokuSolver.guess_least_open_square(self)

    def difficulty (self):
        if not self.solved:
            self.solve()
        self.clues = 0
        # Add up the number of our initial clues through some nifty mapping calls
        map(lambda r: map(lambda i: setattr(self, 'clues', self.clues.__add__(i and 1 or 0)),
                          r),
            self.virgin.grid)
        self.numbers_added = self.group_size ** 2 - self.clues
        rating = DifficultyRating(self.fill_must_fillables,
                                  self.elimination_fillables,
                                  self.guesses,
                                  self.backtraces,
                                  self.numbers_added)
        return rating


class GuessList (list):
    def __init__ (self, *guesses):
        list.__init__(self, *guesses)


    def guesses_for_coord (self, x, y):
        return set([guess.val for guess in filter(lambda guess: guess.x == x and guess.y == y, self)])

    def remove_children (self, guess):
        removed = []
        for g in guess.children:
            if g in self:
                removed.append(g)
                self.remove(g)
        return removed

    def remove_guesses_for_coord (self, x, y):
        nuking = False
        nuked = []
        for i in range(len(self) - 1):
            g = self[i - len(nuked)]
            if g.x == x and g.y == y:
                nuking = True
            if nuking:
                self.remove(g)
                nuked += [g]
        return nuked

class BreadcrumbTrail (GuessList):
    def append (self, guess):
        # Raise an error if we add something to ourselves twice
        if self.guesses_for_coord(guess.x, guess.y):
            raise ValueError("We already have crumbs on %s, %s" % (guess.x, guess.y))
        else:
            list.append(self, guess)

class Guess:
    def __init__ (self, x, y, val):
        self.x = x
        self.y = y
        self.children = []
        self.val = val
        self.consequences = {}

    def add_consequence (self, x, y, val):
        self.consequences[(x, y)] = val

    def __repr__ (self):
        s =  "<Guess (%s, %s)=%s" % (self.x, self.y, self.val)
        if self.consequences:
            s +=   " implies: "
            s += ", ".join(["%s->%s" % (k, v) for k, v in self.consequences.items()])
        s += ">"
        return s


def add_with_diminishing_importance (lst, diminish_by = lambda x: x + 1):
    sum = 0
    for i, n in enumerate(lst):
        sum += float(n) / diminish_by(i)
    return sum

